@@ -27,7 +27,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: io.c,v 1.17 2003/11/14 02:54:03 roland Exp $
+ *	$Id: io.c,v 1.20 2005/06/01 19:22:08 roland Exp $
  */
 
 #include "defs.h"
@@ -78,38 +78,50 @@ struct tcb *tcp;
 void
 tprint_iov(tcp, len, addr)
 struct tcb * tcp;
-int len;
-long addr;
+unsigned long len;
+unsigned long addr;
 {
-	struct iovec *iov;
-	int i;
-
+	struct iovec iov;
+	unsigned long size, cur, end, abbrev_end;
+	int failed = 0;
 
 	if (!len) {
 		tprintf("[]");
 		return;
 	}
-
-	if ((iov = (struct iovec *) malloc(len * sizeof *iov)) == NULL) {
-		fprintf(stderr, "No memory");
+	size = len * sizeof(iov);
+	end = addr + size;
+	if (!verbose(tcp) || size / sizeof(iov) != len || end < addr) {
+		tprintf("%#lx", addr);
 		return;
 	}
-	if (umoven(tcp, addr,
-		   len * sizeof *iov, (char *) iov) < 0) {
-		tprintf("%#lx", tcp->u_arg[1]);
+	if (abbrev(tcp)) {
+		abbrev_end = addr + max_strlen * sizeof(iov);
+		if (abbrev_end < addr)
+			abbrev_end = end;
 	} else {
-		tprintf("[");
-		for (i = 0; i < len; i++) {
-			if (i)
-				tprintf(", ");
-			tprintf("{");
-			printstr(tcp, (long) iov[i].iov_base,
-				iov[i].iov_len);
-			tprintf(", %lu}", (unsigned long)iov[i].iov_len);
-		}
-		tprintf("]");
+		abbrev_end = end;
 	}
-	free((char *) iov);
+	tprintf("[");
+	for (cur = addr; cur < end; cur += sizeof(iov)) {
+		if (cur > addr)
+			tprintf(", ");
+		if (cur >= abbrev_end) {
+			tprintf("...");
+			break;
+		}
+		if (umoven(tcp, cur, sizeof iov, (char *) &iov) < 0) {
+			tprintf("?");
+			failed = 1;
+			break;
+		}
+		tprintf("{");
+		printstr(tcp, (long) iov.iov_base, iov.iov_len);
+		tprintf(", %lu}", (unsigned long)iov.iov_len);
+	}
+	tprintf("]");
+	if (failed)
+		tprintf(" %#lx", addr);
 }
 
 int
@@ -161,8 +173,7 @@ struct tcb *tcp;
 		tprintf(", %lu, %ld", tcp->u_arg[2], tcp->u_arg[3]);
 #else
 		tprintf(", %lu, %llu", tcp->u_arg[2],
-				(((unsigned long long) tcp->u_arg[4]) << 32
-				 | (unsigned) tcp->u_arg[3]));
+			LONG_LONG(tcp->u_arg[3], tcp->u_arg[4]));
 #endif
 	}
 	return 0;
@@ -180,8 +191,7 @@ struct tcb *tcp;
 		tprintf(", %lu, %ld", tcp->u_arg[2], tcp->u_arg[3]);
 #else
 		tprintf(", %lu, %llu", tcp->u_arg[2],
-				(((unsigned long long) tcp->u_arg[4]) << 32
-				 | (unsigned) tcp->u_arg[3]));
+			LONG_LONG(tcp->u_arg[3], tcp->u_arg[4]));
 #endif
 	}
 	return 0;
@@ -198,8 +208,8 @@ struct tcb *tcp;
 {
 	if (entering(tcp)) {
 		tprintf("%ld, %ld, %llu, %lu", tcp->u_arg[0], tcp->u_arg[1],
-			(((unsigned long long) tcp->u_arg[3]) << 32 |
-			 (unsigned) tcp->u_arg[2]), tcp->u_arg[4]);
+			LONG_LONG(tcp->u_arg[2], tcp->u_arg[3]),
+			tcp->u_arg[4]);
 	} else {
 		off_t offset;
 
@@ -353,7 +363,7 @@ int
 sys_ioctl(tcp)
 struct tcb *tcp;
 {
-	struct ioctlent *iop;
+	const struct ioctlent *iop;
 
 	if (entering(tcp)) {
 		tprintf("%ld, ", tcp->u_arg[0]);
