@@ -27,7 +27,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: ipc.c,v 1.7 2004/04/16 21:48:40 roland Exp $
+ *	$Id: ipc.c,v 1.14 2005/12/02 04:34:09 roland Exp $
  */
 
 #include "defs.h"
@@ -67,10 +67,10 @@
 # define IPC_64 0x100
 #endif
 
-extern struct xlat openmodes[];
+extern const struct xlat openmodes[];
 extern void printsigevent(struct tcb *tcp, long arg);
 
-static struct xlat msgctl_flags[] = {
+static const struct xlat msgctl_flags[] = {
 	{ IPC_RMID,	"IPC_RMID"	},
 	{ IPC_SET,	"IPC_SET"	},
 	{ IPC_STAT,	"IPC_STAT"	},
@@ -82,7 +82,7 @@ static struct xlat msgctl_flags[] = {
 	{ 0,		NULL		},
 };
 
-static struct xlat semctl_flags[] = {
+static const struct xlat semctl_flags[] = {
 	{ IPC_RMID,	"IPC_RMID"	},
 	{ IPC_SET,	"IPC_SET"	},
 	{ IPC_STAT,	"IPC_STAT"	},
@@ -101,7 +101,7 @@ static struct xlat semctl_flags[] = {
 	{ 0,		NULL		},
 };
 
-static struct xlat shmctl_flags[] = {
+static const struct xlat shmctl_flags[] = {
 	{ IPC_RMID,	"IPC_RMID"	},
 	{ IPC_SET,	"IPC_SET"	},
 	{ IPC_STAT,	"IPC_STAT"	},
@@ -119,17 +119,23 @@ static struct xlat shmctl_flags[] = {
 	{ 0,		NULL		},
 };
 
-static struct xlat resource_flags[] = {
+static const struct xlat resource_flags[] = {
 	{ IPC_CREAT,	"IPC_CREAT"	},
 	{ IPC_EXCL,	"IPC_EXCL"	},
 	{ IPC_NOWAIT,	"IPC_NOWAIT"	},
+	{ 0,		NULL		},
+};
+
+static const struct xlat shm_resource_flags[] = {
+	{ IPC_CREAT,	"IPC_CREAT"	},
+	{ IPC_EXCL,	"IPC_EXCL"	},
 #ifdef SHM_HUGETLB
 	{ SHM_HUGETLB,	"SHM_HUGETLB"	},
 #endif
 	{ 0,		NULL		},
 };
 
-static struct xlat shm_flags[] = {
+static const struct xlat shm_flags[] = {
 #ifdef LINUX
 	{ SHM_REMAP,	"SHM_REMAP"	},
 #endif /* LINUX */
@@ -138,7 +144,7 @@ static struct xlat shm_flags[] = {
 	{ 0,		NULL		},
 };
 
-static struct xlat msg_flags[] = {
+static const struct xlat msg_flags[] = {
 	{ MSG_NOERROR,	"MSG_NOERROR"	},
 #ifdef LINUX
 	{ MSG_EXCEPT,	"MSG_EXCEPT"	},
@@ -156,7 +162,7 @@ struct tcb *tcp;
 		else
 			tprintf("IPC_PRIVATE");
 		tprintf(", ");
-		if (printflags(resource_flags, tcp->u_arg[1] & ~0777) != 0)
+		if (printflags(resource_flags, tcp->u_arg[1] & ~0777, NULL) != 0)
 			tprintf("|");
 		tprintf("%#lo", tcp->u_arg[1] & 0777);
 	}
@@ -171,17 +177,31 @@ struct tcb *tcp;
 # define PRINTCTL printxval
 #endif
 
+static int
+indirect_ipccall(tcp)
+struct tcb *tcp;
+{
+#ifdef LINUX
+#ifdef X86_64
+	return current_personality > 0;
+#endif
+#if defined IA64
+	return tcp->scno < 1024; /* ia32 emulation syscalls are low */
+#endif
+#if !defined MIPS && !defined HPPA
+	return 1;
+#endif
+#endif	/* LINUX */
+	return 0;
+}
+
 int sys_msgctl(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp)) {
 		tprintf("%lu, ", tcp->u_arg[0]);
 		PRINTCTL(msgctl_flags, tcp->u_arg[1], "MSG_???");
-#ifdef LINUX
-		tprintf(", %#lx", tcp->u_arg[3]);
-#else /* !LINUX */
-		tprintf(", %#lx", tcp->u_arg[2]);
-#endif /* !LINUX */
+		tprintf(", %#lx", tcp->u_arg[indirect_ipccall(tcp) ? 3 : 2]);
 	}
 	return 0;
 }
@@ -193,25 +213,23 @@ struct tcb *tcp;
 
 	if (entering(tcp)) {
 		tprintf("%lu", tcp->u_arg[0]);
-#ifdef LINUX
-		umove(tcp, tcp->u_arg[3], &mtype);
-		tprintf(", {%lu, ", mtype);
-		printstr(tcp, tcp->u_arg[3] + sizeof(long),
-			tcp->u_arg[1]);
-		tprintf("}, %lu", tcp->u_arg[1]);
-		tprintf(", ");
-		if (printflags(msg_flags, tcp->u_arg[2]) == 0)
-			tprintf("0");
-#else /* !LINUX */
-		umove(tcp, tcp->u_arg[1], &mtype);
-		tprintf(", {%lu, ", mtype);
-		printstr(tcp, tcp->u_arg[1] + sizeof(long),
-			tcp->u_arg[2]);
-		tprintf("}, %lu", tcp->u_arg[2]);
-		tprintf(", ");
-		if (printflags(msg_flags, tcp->u_arg[3]) == 0)
-			tprintf("0");
-#endif /* !LINUX */
+		if (indirect_ipccall(tcp)) {
+			umove(tcp, tcp->u_arg[3], &mtype);
+			tprintf(", {%lu, ", mtype);
+			printstr(tcp, tcp->u_arg[3] + sizeof(long),
+				 tcp->u_arg[1]);
+			tprintf("}, %lu", tcp->u_arg[1]);
+			tprintf(", ");
+			printflags(msg_flags, tcp->u_arg[2], "MSG_???");
+		} else {
+			umove(tcp, tcp->u_arg[1], &mtype);
+			tprintf(", {%lu, ", mtype);
+			printstr(tcp, tcp->u_arg[1] + sizeof(long),
+				 tcp->u_arg[2]);
+			tprintf("}, %lu", tcp->u_arg[2]);
+			tprintf(", ");
+			printflags(msg_flags, tcp->u_arg[3], "MSG_???");
+		}
 	}
 	return 0;
 }
@@ -220,38 +238,35 @@ int sys_msgrcv(tcp)
 struct tcb *tcp;
 {
 	long mtype;
-#ifdef LINUX
-	struct ipc_wrapper {
-		struct msgbuf *msgp;
-		long msgtyp;
-	} tmp;
-#endif
 
-
-	if (exiting(tcp)) {
+	if (entering(tcp)) {
+		tprintf("%lu, ", tcp->u_arg[0]);
+	} else {
 		tprintf("%lu", tcp->u_arg[0]);
-#ifdef LINUX
-		umove(tcp, tcp->u_arg[3], &tmp);
-		umove(tcp, (long) tmp.msgp, &mtype);
-		tprintf(", {%lu, ", mtype);
-		printstr(tcp, (long) (tmp.msgp) + sizeof(long),
-			tcp->u_arg[1]);
-		tprintf("}, %lu", tcp->u_arg[1]);
-		tprintf(", %ld", tmp.msgtyp);
-		tprintf(", ");
-		if (printflags(msg_flags, tcp->u_arg[2]) == 0)
-			tprintf("0");
-#else /* !LINUX */
-		umove(tcp, tcp->u_arg[1], &mtype);
-		tprintf(", {%lu, ", mtype);
-		printstr(tcp, tcp->u_arg[1] + sizeof(long),
-			tcp->u_arg[2]);
-		tprintf("}, %lu", tcp->u_arg[2]);
-		tprintf(", %ld", tcp->u_arg[3]);
-		tprintf(", ");
-		if (printflags(msg_flags, tcp->u_arg[4]) == 0)
-			tprintf("0");
-#endif /* !LINUX */
+		if (indirect_ipccall(tcp)) {
+			struct ipc_wrapper {
+				struct msgbuf *msgp;
+				long msgtyp;
+			} tmp;
+			umove(tcp, tcp->u_arg[3], &tmp);
+			umove(tcp, (long) tmp.msgp, &mtype);
+			tprintf(", {%lu, ", mtype);
+			printstr(tcp, (long) (tmp.msgp) + sizeof(long),
+				 tcp->u_arg[1]);
+			tprintf("}, %lu", tcp->u_arg[1]);
+			tprintf(", %ld", tmp.msgtyp);
+			tprintf(", ");
+			printflags(msg_flags, tcp->u_arg[2], "MSG_???");
+		} else {
+			umove(tcp, tcp->u_arg[1], &mtype);
+			tprintf("{%lu, ", mtype);
+			printstr(tcp, tcp->u_arg[1] + sizeof(long),
+				 tcp->u_arg[2]);
+			tprintf("}, %lu", tcp->u_arg[2]);
+			tprintf(", %ld", tcp->u_arg[3]);
+			tprintf(", ");
+			printflags(msg_flags, tcp->u_arg[4], "MSG_???");
+		}
 	}
 	return 0;
 }
@@ -261,13 +276,13 @@ struct tcb *tcp;
 {
 	if (entering(tcp)) {
 		tprintf("%lu", tcp->u_arg[0]);
-#ifdef LINUX
-		tprintf(", %#lx", tcp->u_arg[3]);
-		tprintf(", %lu", tcp->u_arg[1]);
-#else /* !LINUX */
-		tprintf(", %#lx", tcp->u_arg[1]);
-		tprintf(", %lu", tcp->u_arg[2]);
-#endif /* !LINUX */
+		if (indirect_ipccall(tcp)) {
+			tprintf(", %#lx", tcp->u_arg[3]);
+			tprintf(", %lu", tcp->u_arg[1]);
+		} else {
+			tprintf(", %#lx", tcp->u_arg[1]);
+			tprintf(", %lu", tcp->u_arg[2]);
+		}
 	}
 	return 0;
 }
@@ -278,9 +293,15 @@ struct tcb *tcp;
 {
 	if (entering(tcp)) {
 		tprintf("%lu", tcp->u_arg[0]);
-		tprintf(", %#lx", tcp->u_arg[3]);
-		tprintf(", %lu, ", tcp->u_arg[1]);
-		printtv(tcp, tcp->u_arg[5]);
+		if (!indirect_ipccall(tcp)) {
+			tprintf(", %#lx", tcp->u_arg[3]);
+			tprintf(", %lu, ", tcp->u_arg[1]);
+			printtv(tcp, tcp->u_arg[5]);
+		} else {
+			tprintf(", %#lx", tcp->u_arg[1]);
+			tprintf(", %lu, ", tcp->u_arg[2]);
+			printtv(tcp, tcp->u_arg[3]);
+		}
 	}
 	return 0;
 }
@@ -296,7 +317,7 @@ struct tcb *tcp;
 			tprintf("IPC_PRIVATE");
 		tprintf(", %lu", tcp->u_arg[1]);
 		tprintf(", ");
-		if (printflags(resource_flags, tcp->u_arg[2] & ~0777) != 0)
+		if (printflags(resource_flags, tcp->u_arg[2] & ~0777, NULL) != 0)
 			tprintf("|");
 		tprintf("%#lo", tcp->u_arg[2] & 0777);
 	}
@@ -325,7 +346,7 @@ struct tcb *tcp;
 			tprintf("IPC_PRIVATE");
 		tprintf(", %lu", tcp->u_arg[1]);
 		tprintf(", ");
-		if (printflags(resource_flags, tcp->u_arg[2] & ~0777) != 0)
+		if (printflags(shm_resource_flags, tcp->u_arg[2] & ~0777, NULL) != 0)
 			tprintf("|");
 		tprintf("%#lo", tcp->u_arg[2] & 0777);
 	}
@@ -338,11 +359,11 @@ struct tcb *tcp;
 	if (entering(tcp)) {
 		tprintf("%lu, ", tcp->u_arg[0]);
 		PRINTCTL(shmctl_flags, tcp->u_arg[1], "SHM_???");
-#ifdef LINUX
-		tprintf(", %#lx", tcp->u_arg[3]);
-#else /* !LINUX */
-		tprintf(", %#lx", tcp->u_arg[2]);
-#endif /* !LINUX */
+		if (indirect_ipccall(tcp)) {
+			tprintf(", %#lx", tcp->u_arg[3]);
+		} else {
+			tprintf(", %#lx", tcp->u_arg[2]);
+		}
 	}
 	return 0;
 }
@@ -356,17 +377,15 @@ struct tcb *tcp;
 
 	if (exiting(tcp)) {
 		tprintf("%lu", tcp->u_arg[0]);
-#ifdef LINUX
-		tprintf(", %#lx", tcp->u_arg[3]);
-		tprintf(", ");
-		if (printflags(shm_flags, tcp->u_arg[1]) == 0)
-			tprintf("0");
-#else /* !LINUX */
-		tprintf(", %#lx", tcp->u_arg[1]);
-		tprintf(", ");
-		if (printflags(shm_flags, tcp->u_arg[2]) == 0)
-			tprintf("0");
-#endif /* !LINUX */
+		if (indirect_ipccall(tcp)) {
+			tprintf(", %#lx", tcp->u_arg[3]);
+			tprintf(", ");
+			printflags(shm_flags, tcp->u_arg[1], "SHM_???");
+		} else {
+			tprintf(", %#lx", tcp->u_arg[1]);
+			tprintf(", ");
+			printflags(shm_flags, tcp->u_arg[2], "SHM_???");
+		}
 		if (syserror(tcp))
 			return 0;
 #ifdef LINUX
@@ -382,12 +401,13 @@ struct tcb *tcp;
 int sys_shmdt(tcp)
 struct tcb *tcp;
 {
-	if (entering(tcp))
-#ifdef LINUX
-		tprintf("%#lx", tcp->u_arg[3]);
-#else /* !LINUX */
-		tprintf("%#lx", tcp->u_arg[0]);
-#endif /* !LINUX */
+	if (entering(tcp)) {
+		if (indirect_ipccall(tcp)) {
+			tprintf("%#lx", tcp->u_arg[3]);
+		} else {
+			tprintf("%#lx", tcp->u_arg[0]);
+		}
+	}
 	return 0;
 }
 
@@ -401,7 +421,7 @@ struct tcb *tcp;
 		printpath(tcp, tcp->u_arg[0]);
 		tprintf(", ");
 		/* flags */
-		printflags(openmodes, tcp->u_arg[1] + 1);
+		printflags(openmodes, tcp->u_arg[1] + 1, "O_???");
 		if (tcp->u_arg[1] & O_CREAT) {
 # ifndef HAVE_MQUEUE_H
 			tprintf(", %lx", tcp->u_arg[2]);
@@ -471,7 +491,7 @@ long addr;
 			return;
 		}
 		tprintf("{mq_flags=");
-		printflags(openmodes, attr.mq_flags + 1);
+		printflags(openmodes, attr.mq_flags + 1, "O_???");
 		tprintf(", mq_maxmsg=%ld, mq_msgsize=%ld, mq_curmsg=%ld}",
 			attr.mq_maxmsg, attr.mq_msgsize, attr.mq_curmsgs);
 # endif
