@@ -30,12 +30,12 @@
 	{ 0,	0,	sys_setup,		"setup"		}, /* 0 */
 	{ 1,	TP,	sys_exit,		"_exit"		}, /* 1 */
 	{ 0,	TP,	sys_fork,		"fork"		}, /* 2 */
-	{ 3,	0,	sys_read,		"read"		}, /* 3 */
-	{ 3,	0,	sys_write,		"write"		}, /* 4 */
-	{ 3,	TF,	sys_open,		"open"		}, /* 5 */
-	{ 1,	0,	sys_close,		"close"		}, /* 6 */
+	{ 3,	TD,	sys_read,		"read"		}, /* 3 */
+	{ 3,	TD,	sys_write,		"write"		}, /* 4 */
+	{ 3,	TD|TF,	sys_open,		"open"		}, /* 5 */
+	{ 1,	TD,	sys_close,		"close"		}, /* 6 */
 	{ -1,	0,	printargs,		"SYS_7"		}, /* 7 */
-	{ 2,	TF,	sys_creat,		"creat"		}, /* 8 */
+	{ 2,	TD|TF,	sys_creat,		"creat"		}, /* 8 */
 	{ 2,	TF,	sys_link,		"link"		}, /* 9 */
 	{ 1,	TF,	sys_unlink,		"unlink"	}, /* 10 */
 	{ 3,	TF|TP,	sys_execve,		"execve"	}, /* 11 */
@@ -46,7 +46,7 @@
 	{ -1,	0,	printargs,		"SYS_16"	}, /* 16 */
 	{ -1,	0,	printargs,		"SYS_17"	}, /* 17 */
 	{ -1,	0,	printargs,		"SYS_18"	}, /* 18 */
-	{ 3,	0,	sys_lseek,		"lseek"		}, /* 19 */
+	{ 3,	TD,	sys_lseek,		"lseek"		}, /* 19 */
 	{ 0,	0,	sys_getpid,		"getpid"	}, /* 20 */
 	{ 5,	TF,	sys_mount,		"mount"		}, /* 21 */
 	{ 1,	TF,	sys_umount,		"oldumount"	}, /* 22 */
@@ -68,8 +68,8 @@
 	{ 2,	TF,	sys_rename,		"rename"	}, /* 38 */
 	{ 2,	TF,	sys_mkdir,		"mkdir"		}, /* 39 */
 	{ 1,	TF,	sys_rmdir,		"rmdir"		}, /* 40 */
-	{ 1,	0,	sys_dup,		"dup"		}, /* 41 */
-	{ 1,	0,	sys_pipe,		"pipe"		}, /* 42 */
+	{ 1,	TD,	sys_dup,		"dup"		}, /* 41 */
+	{ 1,	TD,	sys_pipe,		"pipe"		}, /* 42 */
 	{ 1,	0,	sys_times,		"times"		}, /* 43 */
 	{ -1,	0,	printargs,		"SYS_44"	}, /* 44 */
 	{ 1,	0,	sys_brk,		"brk"		}, /* 45 */
@@ -81,8 +81,8 @@
 	{ 1,	TF,	sys_acct,		"acct"		}, /* 51 */
 	{ 2,	TF,	sys_umount2,		"umount"	}, /* 52 */
 	{ -1,	0,	printargs,		"SYS_53"	}, /* 53 */
-	{ 3,	0,	sys_ioctl,		"ioctl"		}, /* 54 */
-	{ 3,	0,	sys_fcntl,		"fcntl"		}, /* 55 */
+	{ 3,	TD,	sys_ioctl,		"ioctl"		}, /* 54 */
+	{ 3,	TD,	sys_fcntl,		"fcntl"		}, /* 55 */
 	{ -1,	0,	printargs,		"SYS_56"	}, /* 56 */
 	{ 2,	0,	sys_setpgid,		"setpgid"	}, /* 57 */
 	{ -1,	0,	printargs,		"SYS_58"	}, /* 58 */
@@ -90,7 +90,7 @@
 	{ 1,	0,	sys_umask,		"umask"		}, /* 60 */
 	{ 1,	TF,	sys_chroot,		"chroot"	}, /* 61 */
 	{ 2,	0,	sys_ustat,		"ustat"		}, /* 62 */
-	{ 2,	0,	sys_dup2,		"dup2"		}, /* 63 */
+	{ 2,	TD,	sys_dup2,		"dup2"		}, /* 63 */
 	{ 0,	0,	sys_getppid,		"getppid"	}, /* 64 */
 	{ 0,	0,	sys_getpgrp,		"getpgrp"	}, /* 65 */
 	{ 0,	0,	sys_setsid,		"setsid"	}, /* 66 */
@@ -117,25 +117,25 @@
 	{ 1,	TF,	sys_swapon,		"swapon"	}, /* 87 */
 	{ 3,	0,	sys_reboot,		"reboot"	}, /* 88 */
 	{ -1,	0,	printargs,		"SYS_89"	}, /* 89 */
-	{ 6,	0,	sys_old_mmap,		"mmap"		}, /* 90 */
+	{ 6,	TD,	sys_old_mmap,		"mmap"		}, /* 90 */
 	{ 2,	0,	sys_munmap,		"munmap"	}, /* 91 */
 	{ 2,	TF,	sys_truncate,		"truncate"	}, /* 92 */
-	{ 2,	0,	sys_ftruncate,		"ftruncate"	}, /* 93 */
-	{ 2,	0,	sys_fchmod,		"fchmod"	}, /* 94 */
+	{ 2,	TD,	sys_ftruncate,		"ftruncate"	}, /* 93 */
+	{ 2,	TD,	sys_fchmod,		"fchmod"	}, /* 94 */
 	{ -1,	0,	printargs,		"SYS_95"	}, /* 95 */
 	{ 2,	0,	sys_getpriority,	"getpriority"	}, /* 96 */
 	{ 3,	0,	sys_setpriority,	"setpriority"	}, /* 97 */
 	{ -1,	0,	printargs,		"SYS_98"	}, /* 98 */
 	{ 2,	TF,	sys_statfs,		"statfs"	}, /* 99 */
-	{ 2,	0,	sys_fstatfs,		"fstatfs"	}, /* 100 */
+	{ 2,	TD,	sys_fstatfs,		"fstatfs"	}, /* 100 */
 	{ -1,	0,	printargs,		"SYS_101"	}, /* 101 */
-	{ 2,	0,	sys_socketcall,		"socketcall"	}, /* 102 */
+	{ 2,	TD,	sys_socketcall,		"socketcall"	}, /* 102 */
 	{ 3,	0,	sys_syslog,		"syslog"	}, /* 103 */
 	{ 3,	0,	sys_setitimer,		"setitimer"	}, /* 104 */
 	{ 2,	0,	sys_getitimer,		"getitimer"	}, /* 105 */
 	{ 2,	TF,	sys_stat,		"stat"		}, /* 106 */
 	{ 2,	TF,	sys_lstat,		"lstat"		}, /* 107 */
-	{ 2,	0,	sys_fstat,		"fstat"		}, /* 108 */
+	{ 2,	TD,	sys_fstat,		"fstat"		}, /* 108 */
 	{ -1,	0,	printargs,		"SYS_109"	}, /* 109 */
 	{ -1,	0,	printargs,		"SYS_110"	}, /* 110 */
 	{ 0,	0,	sys_vhangup,		"vhangup"	}, /* 111 */
@@ -145,7 +145,7 @@
 	{ 1,	0,	sys_swapoff,		"swapoff"	}, /* 115 */
 	{ 1,	0,	sys_sysinfo,		"sysinfo"	}, /* 116 */
 	{ 5,	0,	sys_ipc,		"ipc"		}, /* 117 */
-	{ 1,	0,	sys_fsync,		"fsync"		}, /* 118 */
+	{ 1,	TD,	sys_fsync,		"fsync"		}, /* 118 */
 	{ 1,	TS,	sys_sigreturn,		"sigreturn"	}, /* 119 */
 	{ 5,	TP,	sys_clone,		"clone"		}, /* 120 */
 	{ 2,	0,	sys_setdomainname,	"setdomainname"	}, /* 121 */
@@ -155,27 +155,27 @@
 	{ 3,	0,	sys_mprotect,		"mprotect"	}, /* 125 */
 	{ 3,	TS,	sys_sigprocmask,	"sigprocmask"	}, /* 126 */
 	{ 2,	0,	sys_create_module,	"create_module"	}, /* 127 */
-	{ 2,	0,	sys_init_module,	"init_module"	}, /* 128 */
+	{ 3,	0,	sys_init_module,	"init_module"	}, /* 128 */
 	{ 1,	0,	sys_delete_module,	"delete_module"	}, /* 129 */
 	{ 1,	0,	sys_get_kernel_syms,	"get_kernel_syms"}, /* 130 */
 	{ 4,	0,	sys_quotactl,		"quotactl"	}, /* 131 */
 	{ 1,	0,	sys_getpgid,		"getpgid"	}, /* 132 */
-	{ 1,	0,	sys_fchdir,		"fchdir"	}, /* 133 */
+	{ 1,	TD,	sys_fchdir,		"fchdir"	}, /* 133 */
 	{ 0,	0,	sys_bdflush,		"bdflush"	}, /* 134 */
 	{ 3,	0,	sys_sysfs,		"sysfs"		}, /* 135 */
 	{ 1,	0,	sys_personality,	"personality"	}, /* 136 */
 	{ 5,	0,	sys_afs_syscall,	"afs_syscall"	}, /* 137 */
 	{ -1,	0,	printargs,		"SYS_138"	}, /* 138 */
 	{ -1,	0,	printargs,		"SYS_139"	}, /* 139 */
-	{ -1,	0,	printargs,		"SYS_140"	}, /* 140 */
-	{ 3,	0,	sys_getdents,		"getdents"	}, /* 141 */
-	{ 5,	0,	sys_select,		"select"	}, /* 142 */
-	{ 2,	0,	sys_flock,		"flock"		}, /* 143 */
+ 	{ 5,	0,	sys_llseek,		"_llseek"	}, /* 140 */
+	{ 3,	TD,	sys_getdents,		"getdents"	}, /* 141 */
+	{ 5,	TD,	sys_select,		"select"	}, /* 142 */
+	{ 2,	TD,	sys_flock,		"flock"		}, /* 143 */
 	{ 3,	0,	sys_msync,		"msync"		}, /* 144 */
-	{ 3,	0,	sys_readv,		"readv"		}, /* 145 */
-	{ 3,	0,	sys_writev,		"writev"	}, /* 146 */
+	{ 3,	TD,	sys_readv,		"readv"		}, /* 145 */
+	{ 3,	TD,	sys_writev,		"writev"	}, /* 146 */
 	{ 1,	0,	sys_getsid,		"getsid"	}, /* 147 */
-	{ 1,	0,	sys_fdatasync,		"fdatasync"	}, /* 148 */
+	{ 1,	TD,	sys_fdatasync,		"fdatasync"	}, /* 148 */
 	{ 1,	0,	sys_sysctl,		"_sysctl"	}, /* 149 */
 	{ 2,	0,	sys_mlock,		"mlock"		}, /* 150 */
 	{ 2,	0,	sys_munlock,		"munlock"	}, /* 151 */
@@ -195,7 +195,7 @@
 	{ -1,	0,	printargs,		"SYS_165"	}, /* 165 */
 	{ -1,	0,	printargs,		"SYS_166"	}, /* 166 */
 	{ 5,	0,	sys_query_module,	"query_module"	}, /* 167 */
-	{ 3,	0,	sys_poll,		"poll"		}, /* 168 */
+	{ 3,	TD,	sys_poll,		"poll"		}, /* 168 */
 	{ 3,	0,	printargs,		"nfsservctl"	}, /* 169 */
 	{ -1,	0,	printargs,		"SYS_170"	}, /* 170 */
 	{ -1,	0,	printargs,		"SYS_171"	}, /* 171 */
@@ -207,14 +207,14 @@
 	{ 4,	TS,	sys_rt_sigtimedwait,	"rt_sigtimedwait"}, /* 177 */
 	{ 3,	TS,	sys_rt_sigqueueinfo,	"rt_sigqueueinfo"}, /* 178 */
 	{ 2,	TS,	sys_rt_sigsuspend,	"rt_sigsuspend"	}, /* 179 */
-	{ 5,	TF,	sys_pread,		"pread"		}, /* 180 */
-	{ 5,	TF,	sys_pwrite,		"pwrite"	}, /* 181 */
+	{ 5,	TD,	sys_pread,		"pread"		}, /* 180 */
+	{ 5,	TD,	sys_pwrite,		"pwrite"	}, /* 181 */
 	{ -1,	0,	printargs,		"SYS_182"	}, /* 182 */
 	{ 2,	TF,	sys_getcwd,		"getcwd"	}, /* 183 */
 	{ 2,	0,	sys_capget,		"capget"	}, /* 184 */
 	{ 2,	0,	sys_capset,		"capset"	}, /* 185 */
 	{ 2,	TS,	sys_sigaltstack,	"sigaltstack"	}, /* 186 */
-	{ 4,	TF,	sys_sendfile,		"sendfile"	}, /* 187 */
+	{ 4,	TD,	sys_sendfile,		"sendfile"	}, /* 187 */
 	{ 5,	0,	sys_getpmsg,		"getpmsg"	}, /* 188 */
 	{ 5,	0,	sys_putpmsg,		"putpmsg"	}, /* 189 */
 	{ 0,	TP,	sys_vfork,		"vfork"		}, /* 190 */
@@ -234,7 +234,7 @@
 	{ 2,	0,	sys_setregid,		"setregid"	}, /* 204 */
 	{ 2,	0,	sys_getgroups,		"getgroups"	}, /* 205 */
 	{ 2,	0,	sys_setgroups,		"setgroups"	}, /* 206 */
-	{ 3,	0,	sys_fchown,		"fchown"	}, /* 207 */
+	{ 3,	TD,	sys_fchown,		"fchown"	}, /* 207 */
 	{ 3,	0,	sys_setresuid,		"setresuid"	}, /* 208 */
 	{ 3,	0,	sys_getresuid,		"getresuid"	}, /* 209 */
 	{ 3,	0,	sys_setresgid,		"setresgid"	}, /* 210 */
@@ -247,13 +247,13 @@
         { 2,	TF,	sys_pivotroot,		"pivot_root"	}, /* 217 */
 	{ 3,	0,	sys_mincore,	         "mincore"      }, /* 218 */
 	{ 3,	0,	sys_madvise,		"madvise"	}, /* 219 */
-	{ 3,	0,	sys_getdents64,		"getdents64"	}, /* 220 */
+	{ 3,	TD,	sys_getdents64,		"getdents64"	}, /* 220 */
 	{ -1,	0,	printargs,		"SYS_221"	}, /* 221 */
-	{ 4,	0,	sys_readahead,		"readahead"	}, /* 222 */
-	{ 4,	TF,	sys_sendfile64,		"sendfile64"	}, /* 223 */
+	{ 4,	TD,	sys_readahead,		"readahead"	}, /* 222 */
+	{ 4,	TD,	sys_sendfile64,		"sendfile64"	}, /* 223 */
 	{ 5,	TF,	sys_setxattr,		"setxattr"	}, /* 224 */
 	{ 5,	TF,	sys_setxattr,		"lsetxattr"	}, /* 225 */
-	{ 5,	0,	sys_fsetxattr,		"fsetxattr"	}, /* 226 */
+	{ 5,	TD,	sys_fsetxattr,		"fsetxattr"	}, /* 226 */
 	{ 4,	TF,	sys_getxattr,		"getxattr"	}, /* 227 */
 	{ 4,	TF,	sys_getxattr,		"lgetxattr"	}, /* 228 */
 	{ 4,	0,	sys_fgetxattr,		"fgetxattr"	}, /* 229 */
@@ -262,23 +262,23 @@
 	{ 3,	0,	sys_flistxattr,		"flistxattr"	}, /* 232 */
 	{ 2,	TF,	sys_removexattr,	"removexattr"	}, /* 233 */
 	{ 2,	TF,	sys_removexattr,	"lremovexattr"	}, /* 234 */
-	{ 2,	0,	sys_fremovexattr,	"fremovexattr"	}, /* 235 */
+	{ 2,	TD,	sys_fremovexattr,	"fremovexattr"	}, /* 235 */
 	{ 0,	0,	printargs,		"gettid"	}, /* 236 */
 	{ 2,	TS,	sys_kill,		"tkill"		}, /* 237 */
 	{ 5,	0,	sys_futex,		"futex"		}, /* 238 */
 	{ 3,	0,	sys_sched_setaffinity,	"sched_setaffinity" },/* 239 */
 	{ 3,	0,	sys_sched_getaffinity,	"sched_getaffinity" },/* 240 */
-	{ -1,	0,	printargs,		"SYS_241"	}, /* 241 */
+	{ 3,	TS,	sys_tgkill,		"tgkill"	}, /* 241 */
 	{ -1,	0,	printargs,		"SYS_242"	}, /* 242 */
-	{ 2,	0,	printargs,		"io_setup"	}, /* 243 */
-	{ 1,	0,	printargs,		"io_destroy"	}, /* 244 */
-	{ 5,	0,	printargs,		"io_getevents"	}, /* 245 */
-	{ 3,	0,	printargs,		"io_submit"	}, /* 246 */
-	{ 3,	0,	printargs,		"io_cancel"	}, /* 247 */
+	{ 2,	0,	sys_io_setup,		"io_setup"	}, /* 243 */
+	{ 1,	0,	sys_io_destroy,		"io_destroy"	}, /* 244 */
+	{ 5,	0,	sys_io_getevents,	"io_getevents"	}, /* 245 */
+	{ 3,	0,	sys_io_submit,		"io_submit"	}, /* 246 */
+	{ 3,	0,	sys_io_cancel,		"io_cancel"	}, /* 247 */
 	{ 1,	TP,	sys_exit,		"exit_group"	}, /* 248 */
-	{ 1,	0,	printargs,		"epoll_create"	}, /* 249 */
-	{ 4,	0,	printargs,		"epoll_ctl"	}, /* 250 */
-	{ 4,	0,	printargs,		"epoll_wait"	}, /* 251 */
+	{ 1,	0,	sys_epoll_create,	"epoll_create"	}, /* 249 */
+	{ 4,	TD,	sys_epoll_ctl,		"epoll_ctl"	}, /* 250 */
+	{ 4,	TD,	sys_epoll_wait,		"epoll_wait"	}, /* 251 */
 	{ 1,	0,	printargs,		"set_tid_address"}, /* 252 */
 	{ 5,	0,	printargs,		"fadvise64"	}, /* 253 */
 	{ 3,	0,	sys_timer_create,	"timer_create"	}, /* 254 */
@@ -290,31 +290,31 @@
 	{ 2,	0,	sys_clock_gettime,	"clock_gettime"	}, /* 260 */
 	{ 2,	0,	sys_clock_getres,	"clock_getres"	}, /* 261 */
 	{ 4,	0,	sys_clock_nanosleep,	"clock_nanosleep"}, /* 262 */
+	{ 5,	0,	printargs,		"vserver"	}, /* 263 */
+	{ 6,	TF,	sys_fadvise64_64,	"fadvise64_64"	}, /* 264 */
+	{ 3,	TF,	sys_statfs64,		"statfs64"	}, /* 265 */
+	{ 3,	TF,	sys_fstatfs64,		"fstatfs64"	}, /* 266 */
+	{ 5,	0,	sys_remap_file_pages,	"remap_file_pages"}, /* 267 */
+	{ 6,	0,	sys_mbind,		"mbind"		}, /* 268 */
+	{ 5,	0,	sys_get_mempolicy,	"get_mempolicy"	}, /* 269 */
+	{ 3,	0,	sys_set_mempolicy,	"set_mempolicy"	}, /* 270 */
+	{ 4,	0,	sys_mq_open,		"mq_open"	}, /* 271 */
+	{ 1,	0,	sys_mq_unlink,		"mq_unlink"	}, /* 272 */
+	{ 5,	0,	sys_mq_timedsend,	"mq_timedsend"	}, /* 273 */
+	{ 5,	0,	sys_mq_timedreceive,	"mq_timedreceive" }, /* 274 */
+	{ 2,	0,	sys_mq_notify,		"mq_notify"	}, /* 275 */
+	{ 3,	0,	sys_mq_getsetattr,	"mq_getsetattr"	}, /* 276 */
+	{ 5,	0,	printargs,		"sys_kexec_load" }, /* 277 */
+	{ 5,	0,	printargs,		"add_key"	}, /* 278 */
+	{ 4,	0,	printargs,		"request_key"	}, /* 279 */
+	{ 5,	0,	printargs,		"keyctl"	}, /* 280 */
+	{ 5,	TP,	sys_waitid,		"waitid"	}, /* 281 */
+	{ 3,	0,	printargs,		"ioprio_set"	}, /* 282 */
+	{ 2,	0,	printargs,		"ioprio_get"	}, /* 283 */
+	{ 0,	0,	printargs,		"inotify_init"	}, /* 284 */
+	{ 3,	0,	printargs,		"inotify_add_watch" }, /* 285 */
+	{ 2,	0,	printargs,		"inotify_rm_watch" }, /* 286 */
 
-	{ 5,	0,	printargs,		"SYS_263"	}, /* 263 */
-	{ 5,	0,	printargs,		"SYS_264"	}, /* 264 */
-	{ 5,	0,	printargs,		"SYS_265"	}, /* 265 */
-	{ 5,	0,	printargs,		"SYS_266"	}, /* 266 */
-	{ 5,	0,	printargs,		"SYS_267"	}, /* 267 */
-	{ 5,	0,	printargs,		"SYS_268"	}, /* 268 */
-	{ 5,	0,	printargs,		"SYS_269"	}, /* 269 */
-	{ 5,	0,	printargs,		"SYS_270"	}, /* 270 */
-	{ 5,	0,	printargs,		"SYS_271"	}, /* 271 */
-	{ 5,	0,	printargs,		"SYS_272"	}, /* 272 */
-	{ 5,	0,	printargs,		"SYS_273"	}, /* 273 */
-	{ 5,	0,	printargs,		"SYS_274"	}, /* 274 */
-	{ 5,	0,	printargs,		"SYS_275"	}, /* 275 */
-	{ 5,	0,	printargs,		"SYS_276"	}, /* 276 */
-	{ 5,	0,	printargs,		"SYS_277"	}, /* 277 */
-	{ 5,	0,	printargs,		"SYS_278"	}, /* 278 */
-	{ 5,	0,	printargs,		"SYS_279"	}, /* 279 */
-	{ 5,	0,	printargs,		"SYS_280"	}, /* 280 */
-	{ 5,	0,	printargs,		"SYS_281"	}, /* 281 */
-	{ 5,	0,	printargs,		"SYS_282"	}, /* 282 */
-	{ 5,	0,	printargs,		"SYS_283"	}, /* 283 */
-	{ 5,	0,	printargs,		"SYS_284"	}, /* 284 */
-	{ 5,	0,	printargs,		"SYS_285"	}, /* 285 */
-	{ 5,	0,	printargs,		"SYS_286"	}, /* 286 */
 	{ 5,	0,	printargs,		"SYS_287"	}, /* 287 */
 	{ 5,	0,	printargs,		"SYS_288"	}, /* 288 */
 	{ 5,	0,	printargs,		"SYS_289"	}, /* 289 */
