@@ -34,7 +34,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: process.c,v 1.84 2004/06/04 01:50:45 roland Exp $
+ *	$Id: process.c,v 1.98 2005/08/03 11:23:43 roland Exp $
  */
 
 #include "defs.h"
@@ -57,17 +57,17 @@
 #endif
 
 #if HAVE_ASM_REG_H
-#ifdef SPARC
+#if defined (SPARC) || defined (SPARC64)
 #  define fpq kernel_fpq
 #  define fq kernel_fq
 #  define fpu kernel_fpu
-#endif
+#endif /* SPARC || SPARC64 */
 #include <asm/reg.h>
-#ifdef SPARC
+#if defined (SPARC) || defined (SPARC64)
 #  undef fpq
 #  undef fq
 #  undef fpu
-#endif
+#endif /* SPARC || SPARC64 */
 #endif /* HAVE_ASM_REG_H */
 
 #ifdef HAVE_SYS_REG_H
@@ -93,6 +93,14 @@
 # undef pt_all_user_regs
 #endif
 
+#if defined (LINUX) && defined (SPARC64)
+# define r_pc r_tpc
+# undef PTRACE_GETREGS
+# define PTRACE_GETREGS PTRACE_GETREGS64
+# undef PTRACE_SETREGS
+# define PTRACE_SETREGS PTRACE_SETREGS64
+#endif /* LINUX && SPARC64 */
+
 #ifdef HAVE_LINUX_FUTEX_H
 #include <linux/futex.h>
 #endif
@@ -135,7 +143,7 @@
 
 /* WTA: this was `&& !defined(LINUXSPARC)', this seems unneeded though? */
 #if defined(HAVE_PRCTL)
-static struct xlat prctl_options[] = {
+static const struct xlat prctl_options[] = {
 #ifdef PR_MAXPROCS
 	{ PR_MAXPROCS,		"PR_MAXPROCS"		},
 #endif
@@ -212,7 +220,7 @@ static struct xlat prctl_options[] = {
 };
 
 
-const char *
+static const char *
 unalignctl_string (unsigned int ctl)
 {
 	static char buf[16];
@@ -388,7 +396,7 @@ struct tcb *tcp;
 				tcp->flags |= TCB_GROUP_EXITING;
 		} else
 # endif
-		if (tcp->scno == __NR_exit_group)
+		if (known_scno(tcp) == __NR_exit_group)
 			tcp->flags |= TCB_GROUP_EXITING;
 #endif
 	}
@@ -455,7 +463,7 @@ struct tcb *tcp;
 
 	if (exiting(tcp)) {
 #ifdef SYS_rfork
-		if (tcp->scno == SYS_rfork && !(tcp->u_arg[0]&RFPROC))
+		if (known_scno(tcp) == SYS_rfork && !(tcp->u_arg[0]&RFPROC))
 			return 0;
 #endif
 		if (getrval2(tcp))
@@ -502,7 +510,7 @@ struct tcb *tcp;
 #define CLONE_UNTRACED		0x00800000	/* set if the tracing process can't force CLONE_PTRACE on this clone */
 #define CLONE_CHILD_SETTID	0x01000000	/* set the TID in the child */
 
-static struct xlat clone_flags[] = {
+static const struct xlat clone_flags[] = {
     { CLONE_VM,		"CLONE_VM"	},
     { CLONE_FS,		"CLONE_FS"	},
     { CLONE_FILES,	"CLONE_FILES"	},
@@ -534,10 +542,10 @@ extern void print_ldt_entry();
 # if defined IA64
 #  define ARG_FLAGS	0
 #  define ARG_STACK	1
-#  define ARG_STACKSIZE	(tcp->scno == SYS_clone2 ? 2 : -1)
-#  define ARG_PTID	(tcp->scno == SYS_clone2 ? 3 : 2)
-#  define ARG_CTID	(tcp->scno == SYS_clone2 ? 4 : 3)
-#  define ARG_TLS	(tcp->scno == SYS_clone2 ? 5 : 4)
+#  define ARG_STACKSIZE	(known_scno(tcp) == SYS_clone2 ? 2 : -1)
+#  define ARG_PTID	(known_scno(tcp) == SYS_clone2 ? 3 : 2)
+#  define ARG_CTID	(known_scno(tcp) == SYS_clone2 ? 4 : 3)
+#  define ARG_TLS	(known_scno(tcp) == SYS_clone2 ? 5 : 4)
 # elif defined S390 || defined S390X
 #  define ARG_STACK	0
 #  define ARG_FLAGS	1
@@ -571,8 +579,7 @@ struct tcb *tcp;
 				tcp->u_arg[ARG_STACKSIZE]);
 # endif
 		tprintf("flags=");
-		if (printflags(clone_flags, flags &~ CSIGNAL) == 0)
-			tprintf("0");
+		printflags(clone_flags, flags &~ CSIGNAL, NULL);
 		if ((flags & CSIGNAL) != 0)
 			tprintf("|%s", signame(flags & CSIGNAL));
 		if ((flags & (CLONE_PARENT_SETTID|CLONE_CHILD_SETTID
@@ -641,7 +648,7 @@ int new;
 	if (ptrace(PTRACE_POKEUSER, tcp->pid, (char*)(4*PT_ORIG_D0), new)<0)
 	    	return -1;
 	return 0;
-#elif defined(SPARC)
+#elif defined(SPARC) || defined(SPARC64)
 	struct regs regs;
 	if (ptrace(PTRACE_GETREGS, tcp->pid, (char*)&regs, 0)<0)
 		return -1;
@@ -694,6 +701,7 @@ int new;
 	return -1;
 }
 
+#if 0
 int
 setarg(tcp, argnum)
 	struct tcb *tcp;
@@ -771,6 +779,7 @@ setarg(tcp, argnum)
 #endif
 	return 0;
 }
+#endif
 
 #if defined SYS_clone || defined SYS_clone2
 int
@@ -924,7 +933,7 @@ struct tcb *tcp;
 	int dont_follow = 0;
 
 #ifdef SYS_vfork
-	if (tcp->scno == SYS_vfork) {
+	if (known_scno(tcp) == SYS_vfork) {
 		/* Attempt to make vfork into fork, which we can follow. */
 		if (!followvfork ||
 		    change_syscall(tcp, SYS_fork) < 0)
@@ -1218,34 +1227,53 @@ int
 sys_setgroups(tcp)
 struct tcb *tcp;
 {
-	int i, len;
-	GETGROUPS_T *gidset;
-
 	if (entering(tcp)) {
+		unsigned long len, size, start, cur, end, abbrev_end;
+		GETGROUPS_T gid;
+		int failed = 0;
+
 		len = tcp->u_arg[0];
-		tprintf("%u, ", len);
-		if (len <= 0) {
+		tprintf("%lu, ", len);
+		if (len == 0) {
 			tprintf("[]");
 			return 0;
 		}
-		gidset = (GETGROUPS_T *) malloc(len * sizeof(GETGROUPS_T));
-		if (gidset == NULL) {
-			fprintf(stderr, "sys_setgroups: out of memory\n");
-			return -1;
+		start = tcp->u_arg[1];
+		if (start == 0) {
+			tprintf("NULL");
+			return 0;
 		}
-		if (!verbose(tcp))
-			tprintf("%#lx", tcp->u_arg[1]);
-		else if (umoven(tcp, tcp->u_arg[1],
-		    len * sizeof(GETGROUPS_T), (char *) gidset) < 0)
-			tprintf("[?]");
-		else {
-			tprintf("[");
-			for (i = 0; i < len; i++)
-				tprintf("%s%lu", i ? ", " : "",
-					(unsigned long) gidset[i]);
-			tprintf("]");
+		size = len * sizeof(gid);
+		end = start + size;
+		if (!verbose(tcp) || size / sizeof(gid) != len || end < start) {
+			tprintf("%#lx", start);
+			return 0;
 		}
-		free((char *) gidset);
+		if (abbrev(tcp)) {
+			abbrev_end = start + max_strlen * sizeof(gid);
+			if (abbrev_end < start)
+				abbrev_end = end;
+		} else {
+			abbrev_end = end;
+		}
+		tprintf("[");
+		for (cur = start; cur < end; cur += sizeof(gid)) {
+			if (cur > start)
+				tprintf(", ");
+			if (cur >= abbrev_end) {
+				tprintf("...");
+				break;
+			}
+			if (umoven(tcp, cur, sizeof(gid), (char *) &gid) < 0) {
+				tprintf("?");
+				failed = 1;
+				break;
+			}
+			tprintf("%lu", (unsigned long) gid);
+		}
+		tprintf("]");
+		if (failed)
+			tprintf(" %#lx", tcp->u_arg[1]);
 	}
 	return 0;
 }
@@ -1254,38 +1282,62 @@ int
 sys_getgroups(tcp)
 struct tcb *tcp;
 {
-	int i, len;
-	GETGROUPS_T *gidset;
+	unsigned long len;
 
 	if (entering(tcp)) {
 		len = tcp->u_arg[0];
-		tprintf("%u, ", len);
+		tprintf("%lu, ", len);
 	} else {
+		unsigned long size, start, cur, end, abbrev_end;
+		GETGROUPS_T gid;
+		int failed = 0;
+
 		len = tcp->u_rval;
-		if (len <= 0) {
+		if (len == 0) {
 			tprintf("[]");
 			return 0;
 		}
-		gidset = (GETGROUPS_T *) malloc(len * sizeof(GETGROUPS_T));
-		if (gidset == NULL) {
-			fprintf(stderr, "sys_getgroups: out of memory\n");
-			return -1;
-		}
-		if (!tcp->u_arg[1])
+		start = tcp->u_arg[1];
+		if (start == 0) {
 			tprintf("NULL");
-		else if (!verbose(tcp) || tcp->u_arg[0] == 0)
-			tprintf("%#lx", tcp->u_arg[1]);
-		else if (umoven(tcp, tcp->u_arg[1],
-		    len * sizeof(GETGROUPS_T), (char *) gidset) < 0)
-			tprintf("[?]");
-		else {
-			tprintf("[");
-			for (i = 0; i < len; i++)
-				tprintf("%s%lu", i ? ", " : "",
-					(unsigned long) gidset[i]);
-			tprintf("]");
+			return 0;
 		}
-		free((char *)gidset);
+		if (tcp->u_arg[0] == 0) {
+			tprintf("%#lx", start);
+			return 0;
+		}
+		size = len * sizeof(gid);
+		end = start + size;
+		if (!verbose(tcp) || tcp->u_arg[0] == 0 ||
+		    size / sizeof(gid) != len || end < start) {
+			tprintf("%#lx", start);
+			return 0;
+		}
+		if (abbrev(tcp)) {
+			abbrev_end = start + max_strlen * sizeof(gid);
+			if (abbrev_end < start)
+				abbrev_end = end;
+		} else {
+			abbrev_end = end;
+		}
+		tprintf("[");
+		for (cur = start; cur < end; cur += sizeof(gid)) {
+			if (cur > start)
+				tprintf(", ");
+			if (cur >= abbrev_end) {
+				tprintf("...");
+				break;
+			}
+			if (umoven(tcp, cur, sizeof(gid), (char *) &gid) < 0) {
+				tprintf("?");
+				failed = 1;
+				break;
+			}
+			tprintf("%lu", (unsigned long) gid);
+		}
+		tprintf("]");
+		if (failed)
+			tprintf(" %#lx", tcp->u_arg[1]);
 	}
 	return 0;
 }
@@ -1295,34 +1347,53 @@ int
 sys_setgroups32(tcp)
 struct tcb *tcp;
 {
-	int i, len;
-	GETGROUPS32_T *gidset;
-
 	if (entering(tcp)) {
+		unsigned long len, size, start, cur, end, abbrev_end;
+		GETGROUPS32_T gid;
+		int failed = 0;
+
 		len = tcp->u_arg[0];
-		tprintf("%u, ", len);
-		if (len <= 0) {
+		tprintf("%lu, ", len);
+		if (len == 0) {
 			tprintf("[]");
 			return 0;
 		}
-		gidset = (GETGROUPS32_T *) malloc(len * sizeof(GETGROUPS32_T));
-		if (gidset == NULL) {
-			fprintf(stderr, "sys_setgroups32: out of memory\n");
-			return -1;
+		start = tcp->u_arg[1];
+		if (start == 0) {
+			tprintf("NULL");
+			return 0;
 		}
-		if (!verbose(tcp))
-			tprintf("%#lx", tcp->u_arg[1]);
-		else if (umoven(tcp, tcp->u_arg[1],
-		    len * sizeof(GETGROUPS32_T), (char *) gidset) < 0)
-			tprintf("[?]");
-		else {
-			tprintf("[");
-			for (i = 0; i < len; i++)
-				tprintf("%s%lu", i ? ", " : "",
-					(unsigned long) gidset[i]);
-			tprintf("]");
+		size = len * sizeof(gid);
+		end = start + size;
+		if (!verbose(tcp) || size / sizeof(gid) != len || end < start) {
+			tprintf("%#lx", start);
+			return 0;
 		}
-		free((char *) gidset);
+		if (abbrev(tcp)) {
+			abbrev_end = start + max_strlen * sizeof(gid);
+			if (abbrev_end < start)
+				abbrev_end = end;
+		} else {
+			abbrev_end = end;
+		}
+		tprintf("[");
+		for (cur = start; cur < end; cur += sizeof(gid)) {
+			if (cur > start)
+				tprintf(", ");
+			if (cur >= abbrev_end) {
+				tprintf("...");
+				break;
+			}
+			if (umoven(tcp, cur, sizeof(gid), (char *) &gid) < 0) {
+				tprintf("?");
+				failed = 1;
+				break;
+			}
+			tprintf("%lu", (unsigned long) gid);
+		}
+		tprintf("]");
+		if (failed)
+			tprintf(" %#lx", tcp->u_arg[1]);
 	}
 	return 0;
 }
@@ -1331,38 +1402,58 @@ int
 sys_getgroups32(tcp)
 struct tcb *tcp;
 {
-	int i, len;
-	GETGROUPS32_T *gidset;
+	unsigned long len;
 
 	if (entering(tcp)) {
 		len = tcp->u_arg[0];
-		tprintf("%u, ", len);
+		tprintf("%lu, ", len);
 	} else {
+		unsigned long size, start, cur, end, abbrev_end;
+		GETGROUPS32_T gid;
+		int failed = 0;
+
 		len = tcp->u_rval;
-		if (len <= 0) {
+		if (len == 0) {
 			tprintf("[]");
 			return 0;
 		}
-		gidset = (GETGROUPS32_T *) malloc(len * sizeof(GETGROUPS32_T));
-		if (gidset == NULL) {
-			fprintf(stderr, "sys_getgroups32: out of memory\n");
-			return -1;
-		}
-		if (!tcp->u_arg[1])
+		start = tcp->u_arg[1];
+		if (start == 0) {
 			tprintf("NULL");
-		else if (!verbose(tcp) || tcp->u_arg[0] == 0)
-			tprintf("%#lx", tcp->u_arg[1]);
-		else if (umoven(tcp, tcp->u_arg[1],
-		    len * sizeof(GETGROUPS32_T), (char *) gidset) < 0)
-			tprintf("[?]");
-		else {
-			tprintf("[");
-			for (i = 0; i < len; i++)
-				tprintf("%s%lu", i ? ", " : "",
-					(unsigned long) gidset[i]);
-			tprintf("]");
+			return 0;
 		}
-		free((char *)gidset);
+		size = len * sizeof(gid);
+		end = start + size;
+		if (!verbose(tcp) || tcp->u_arg[0] == 0 ||
+		    size / sizeof(gid) != len || end < start) {
+			tprintf("%#lx", start);
+			return 0;
+		}
+		if (abbrev(tcp)) {
+			abbrev_end = start + max_strlen * sizeof(gid);
+			if (abbrev_end < start)
+				abbrev_end = end;
+		} else {
+			abbrev_end = end;
+		}
+		tprintf("[");
+		for (cur = start; cur < end; cur += sizeof(gid)) {
+			if (cur > start)
+				tprintf(", ");
+			if (cur >= abbrev_end) {
+				tprintf("...");
+				break;
+			}
+			if (umoven(tcp, cur, sizeof(gid), (char *) &gid) < 0) {
+				tprintf("?");
+				failed = 1;
+				break;
+			}
+			tprintf("%lu", (unsigned long) gid);
+		}
+		tprintf("]");
+		if (failed)
+			tprintf(" %#lx", tcp->u_arg[1]);
 	}
 	return 0;
 }
@@ -1434,7 +1525,7 @@ struct tcb *tcp;
 #include <sys/privilege.h>
 
 
-static struct xlat procpriv_cmds [] = {
+static const struct xlat procpriv_cmds [] = {
 	{ SETPRV,	"SETPRV"	},
 	{ CLRPRV,	"CLRPRV"	},
 	{ PUTPRV,	"PUTPRV"	},
@@ -1444,7 +1535,7 @@ static struct xlat procpriv_cmds [] = {
 };
 
 
-static struct xlat procpriv_priv [] = {
+static const struct xlat procpriv_priv [] = {
 	{ P_OWNER,	"P_OWNER"	},
 	{ P_AUDIT,	"P_AUDIT"	},
 	{ P_COMPAT,	"P_COMPAT"	},
@@ -1477,7 +1568,7 @@ static struct xlat procpriv_priv [] = {
 };
 
 
-static struct xlat procpriv_type [] = {
+static const struct xlat procpriv_type [] = {
 	{ PS_FIX,	"PS_FIX"	},
 	{ PS_INH,	"PS_INH"	},
 	{ PS_MAX,	"PS_MAX"	},
@@ -1491,7 +1582,7 @@ printpriv(tcp, addr, len, opt)
 struct tcb *tcp;
 long addr;
 int len;
-struct xlat *opt;
+const struct xlat *opt;
 {
 	priv_t buf [128];
 	int max = verbose (tcp) ? sizeof buf / sizeof buf [0] : 10;
@@ -1567,39 +1658,6 @@ struct tcb *tcp;
 #endif
 
 
-void
-fake_execve(tcp, program, argv, envp)
-struct tcb *tcp;
-char *program;
-char *argv[];
-char *envp[];
-{
-	int i;
-
-#ifdef ARM
-	if (!(qual_flags[SYS_execve - __NR_SYSCALL_BASE] & QUAL_TRACE))
-		return;
-#else
-	if (!(qual_flags[SYS_execve] & QUAL_TRACE))
-		return;
-#endif /* !ARM */
-	printleader(tcp);
-	tprintf("execve(");
-	string_quote(program);
-	tprintf(", [");
-	for (i = 0; argv[i] != NULL; i++) {
-		if (i != 0)
-			tprintf(", ");
-		string_quote(argv[i]);
-	}
-	for (i = 0; envp[i] != NULL; i++)
-		;
-	tprintf("], [/* %d var%s */]) ", i, (i != 1) ? "s" : "");
-	tabto(acolumn);
-	tprintf("= 0");
-	printtrailer(tcp);
-}
-
 static void
 printargv(tcp, addr)
 struct tcb *tcp;
@@ -1689,12 +1747,6 @@ struct tcb *tcp;
 			tprintf("]");
 		}
 	}
-#if defined LINUX && defined TCB_WAITEXECVE
-	if (exiting(tcp) && syserror(tcp))
-		tcp->flags &= ~TCB_WAITEXECVE;
-	else
-		tcp->flags |= TCB_WAITEXECVE;
-#endif /* LINUX && TCB_WAITEXECVE */
 	return 0;
 }
 
@@ -1720,6 +1772,12 @@ struct tcb *tcp;
 	if (exiting(tcp) && !syserror(tcp) && followfork)
 		fixvfork(tcp);
 #endif /* SUNOS4 */
+#if defined LINUX && defined TCB_WAITEXECVE
+	if (exiting(tcp) && syserror(tcp))
+		tcp->flags &= ~TCB_WAITEXECVE;
+	else
+		tcp->flags |= TCB_WAITEXECVE;
+#endif /* LINUX && TCB_WAITEXECVE */
 	return 0;
 }
 
@@ -1735,7 +1793,7 @@ struct tcb *tcp;
 #endif
 #endif /* LINUX */
 
-static struct xlat wait4_options[] = {
+static const struct xlat wait4_options[] = {
 	{ WNOHANG,	"WNOHANG"	},
 #ifndef WSTOPPED
 	{ WUNTRACED,	"WUNTRACED"	},
@@ -1767,6 +1825,20 @@ static struct xlat wait4_options[] = {
 	{ 0,		NULL		},
 };
 
+#if !defined WCOREFLAG && defined WCOREFLG
+# define WCOREFLAG WCOREFLG
+#endif
+#ifndef WCOREFLAG
+#define WCOREFLAG 0x80
+#endif
+
+#ifndef W_STOPCODE
+#define W_STOPCODE(sig)		((sig) << 8 | 0x7f)
+#endif
+#ifndef W_EXITCODE
+#define W_EXITCODE(ret, sig)	((ret) << 8 | (sig))
+#endif
+
 static int
 printstatus(status)
 int status;
@@ -1831,8 +1903,7 @@ int bitness;
 			exited = printstatus(status);
 		/* options */
 		tprintf(", ");
-		if (!printflags(wait4_options, tcp->u_arg[2]))
-			tprintf("0");
+		printflags(wait4_options, tcp->u_arg[2], "W???");
 		if (n == 4) {
 			tprintf(", ");
 			/* usage */
@@ -1860,8 +1931,9 @@ int bitness;
 }
 
 int
-internal_wait(tcp)
+internal_wait(tcp, flagarg)
 struct tcb *tcp;
+int flagarg;
 {
 	int got_kids;
 
@@ -1887,7 +1959,7 @@ struct tcb *tcp;
 
 		/* ??? WTA: fix bug with hanging children */
 
-		if (!(tcp->u_arg[2] & WNOHANG)) {
+		if (!(tcp->u_arg[flagarg] & WNOHANG)) {
 			/*
 			 * There are traced children.  We'll make the parent
 			 * block to avoid a false ECHILD error due to our
@@ -1916,7 +1988,8 @@ struct tcb *tcp;
 					    (tcp->flags & TCB_CLONE_THREAD)
 					    ? tcp->parent :
 #endif
-					    tcp))
+					    tcp) ||
+				    (child->flags & TCB_EXITING))
 					return 0;
 			}
 			tcp->flags |= TCB_SUSPENDED;
@@ -1928,7 +2001,7 @@ struct tcb *tcp;
 		}
 	}
 	if (exiting(tcp) && tcp->u_error == ECHILD && got_kids) {
-		if (tcp->u_arg[2] & WNOHANG) {
+		if (tcp->u_arg[flagarg] & WNOHANG) {
 			/* We must force a fake result of 0 instead of
 			   the ECHILD error.  */
 			extern int force_result();
@@ -2004,16 +2077,26 @@ struct tcb *tcp;
 }
 #endif
 
-#ifdef SVR4
+#if defined SVR4 || defined LINUX
 
-static struct xlat waitid_types[] = {
+static const struct xlat waitid_types[] = {
 	{ P_PID,	"P_PID"		},
+#ifdef P_PPID
 	{ P_PPID,	"P_PPID"	},
+#endif
 	{ P_PGID,	"P_PGID"	},
+#ifdef P_SID
 	{ P_SID,	"P_SID"		},
+#endif
+#ifdef P_CID
 	{ P_CID,	"P_CID"		},
+#endif
+#ifdef P_UID
 	{ P_UID,	"P_UID"		},
+#endif
+#ifdef P_GID
 	{ P_GID,	"P_GID"		},
+#endif
 	{ P_ALL,	"P_ALL"		},
 #ifdef P_LWPID
 	{ P_LWPID,	"P_LWPID"	},
@@ -2031,11 +2114,6 @@ struct tcb *tcp;
 	if (entering(tcp)) {
 		printxval(waitid_types, tcp->u_arg[0], "P_???");
 		tprintf(", %ld, ", tcp->u_arg[1]);
-		if (tcp->nchildren > 0) {
-			/* There are traced children */
-			tcp->flags |= TCB_SUSPENDED;
-			tcp->waitpid = tcp->u_arg[0];
-		}
 	}
 	else {
 		/* siginfo */
@@ -2050,13 +2128,22 @@ struct tcb *tcp;
 			printsiginfo(&si, verbose (tcp));
 		/* options */
 		tprintf(", ");
-		if (!printflags(wait4_options, tcp->u_arg[3]))
-			tprintf("0");
+		printflags(wait4_options, tcp->u_arg[3], "W???");
+		if (tcp->u_nargs > 4) {
+			/* usage */
+			tprintf(", ");
+			if (!tcp->u_arg[4])
+				tprintf("NULL");
+			else if (tcp->u_error)
+				tprintf("%#lx", tcp->u_arg[4]);
+			else
+				printrusage(tcp, tcp->u_arg[4]);
+		}
 	}
 	return 0;
 }
 
-#endif /* SVR4 */
+#endif /* SVR4 or LINUX */
 
 int
 sys_alarm(tcp)
@@ -2101,7 +2188,7 @@ struct tcb *tcp;
 
 #ifndef SVR4
 
-static struct xlat ptrace_cmds[] = {
+static const struct xlat ptrace_cmds[] = {
 #ifndef FREEBSD
 	{ PTRACE_TRACEME,	"PTRACE_TRACEME"	},
 	{ PTRACE_PEEKTEXT,	"PTRACE_PEEKTEXT",	},
@@ -2133,6 +2220,12 @@ static struct xlat ptrace_cmds[] = {
 #ifdef PTRACE_SETFPXREGS
 	{ PTRACE_SETFPXREGS,	"PTRACE_SETFPXREGS",	},
 #endif
+#ifdef PTRACE_GETVRREGS
+	{ PTRACE_GETVRREGS,	"PTRACE_GETVRREGS",	},
+#endif
+#ifdef PTRACE_SETVRREGS
+	{ PTRACE_SETVRREGS,	"PTRACE_SETVRREGS",	},
+#endif
 #ifdef SUNOS4
 	{ PTRACE_READDATA,	"PTRACE_READDATA"	},
 	{ PTRACE_WRITEDATA,	"PTRACE_WRITEDATA"	},
@@ -2190,7 +2283,7 @@ static struct xlat ptrace_cmds[] = {
 #ifndef SUNOS4_KERNEL_ARCH_KLUDGE
 static
 #endif /* !SUNOS4_KERNEL_ARCH_KLUDGE */
-struct xlat struct_user_offsets[] = {
+const struct xlat struct_user_offsets[] = {
 #ifdef LINUX
 #if defined(S390) || defined(S390X)
 	{ PT_PSWMASK,		"psw_mask"				},
@@ -2497,33 +2590,33 @@ struct xlat struct_user_offsets[] = {
 	{ 4*SS,			"4*SS"					},
 #else /* !I386 */
 #ifdef X86_64
-	{ 8*RDI,		"8*RDI"					},
-	{ 8*RSI,		"8*RSI"					},
-	{ 8*RDX,		"8*RDX"					},
-	{ 8*R10, 		"8*R10" },
-	{ 8*R8, 		"8*R8" },
-	{ 8*R9, 		"8*R9" },
-	{ 8*RBX,		"8*RBX"					},
-	{ 8*RCX,		"8*RCX"					},
+	{ 8*R15, 		"8*R15"					},
+	{ 8*R14, 		"8*R14"					},
+	{ 8*R13, 		"8*R13"					},
+	{ 8*R12, 		"8*R12"					},
 	{ 8*RBP,		"8*RBP"					},
+	{ 8*RBX,		"8*RBX"					},
+	{ 8*R11,		"8*R11"					},
+	{ 8*R10,		"8*R10"					},
+	{ 8*R9,			"8*R9"					},
+	{ 8*R8,			"8*R8"					},
 	{ 8*RAX,		"8*RAX"					},
+	{ 8*RCX,		"8*RCX"					},
+	{ 8*RDX,		"8*RDX"					},
+	{ 8*RSI,		"8*RSI"					},
+	{ 8*RDI,		"8*RDI"					},
 #if 0
-	{ 8*DS,			"8*DS"					},
-	{ 8*ES,			"8*ES"					},
-	{ 8*FS,			"8*FS"					},
-	{ 8*GS,			"8*GS"					},
+	{ DS,			"DS"					},
+	{ ES,			"ES"					},
+	{ FS,			"FS"					},
+	{ GS,			"GS"					},
 #endif
-	{ 8*ORIG_RAX,		"8*ORIG_EAX"				},
+	{ 8*ORIG_RAX,		"8*ORIG_RAX"				},
 	{ 8*RIP,		"8*RIP"					},
 	{ 8*CS,			"8*CS"					},
 	{ 8*EFLAGS,		"8*EFL"					},
-	{ 8*RSP,		"8*RSP"				},
+	{ 8*RSP,		"8*RSP"					},
 	{ 8*SS,			"8*SS"					},
-	{ 8*R11, 		"8*R11" },
-	{ 8*R12, 		"8*R12" },
-	{ 8*R13, 		"8*R13" },
-	{ 8*R14, 		"8*R14" },
-	{ 8*R15, 		"8*R15" },
 #endif
 #ifdef M68K
 	{ 4*PT_D1,		"4*PT_D1"				},
@@ -2774,7 +2867,7 @@ struct xlat struct_user_offsets[] = {
 	{ uoff(regs.ARM_cpsr),	"cpsr"					},
 #endif
 
-#if !defined(S390) && !defined(S390X) && !defined(MIPS)
+#if !defined(S390) && !defined(S390X) && !defined(MIPS) && !defined(SPARC64)
 	{ uoff(u_fpvalid),	"offsetof(struct user, u_fpvalid)"	},
 #endif
 #if  defined(I386) || defined(X86_64)
@@ -2787,17 +2880,23 @@ struct xlat struct_user_offsets[] = {
 	{ uoff(u_tsize),	"offsetof(struct user, u_tsize)"	},
 	{ uoff(u_dsize),	"offsetof(struct user, u_dsize)"	},
 	{ uoff(u_ssize),	"offsetof(struct user, u_ssize)"	},
+#if !defined(SPARC64)
 	{ uoff(start_code),	"offsetof(struct user, start_code)"	},
+#endif
 #ifdef SH64
 	{ uoff(start_data),	"offsetof(struct user, start_data)"	},
 #endif
+#if !defined(SPARC64)
 	{ uoff(start_stack),	"offsetof(struct user, start_stack)"	},
+#endif
 	{ uoff(signal),		"offsetof(struct user, signal)"		},
-#if !defined(S390) && !defined(S390X) && !defined(MIPS) && !defined(SH) && !defined(SH64)
+#if !defined(S390) && !defined(S390X) && !defined(MIPS) && !defined(SH) && !defined(SH64) && !defined(SPARC64)
 	{ uoff(reserved),	"offsetof(struct user, reserved)"	},
 #endif
+#if !defined(SPARC64)
 	{ uoff(u_ar0),		"offsetof(struct user, u_ar0)"		},
-#if !defined(ARM) && !defined(MIPS) && !defined(S390) && !defined(S390X)
+#endif
+#if !defined(ARM) && !defined(MIPS) && !defined(S390) && !defined(S390X) && !defined(SPARC64)
 	{ uoff(u_fpstate),	"offsetof(struct user, u_fpstate)"	},
 #endif
 	{ uoff(magic),		"offsetof(struct user, magic)"		},
@@ -2867,7 +2966,7 @@ int
 sys_ptrace(tcp)
 struct tcb *tcp;
 {
-	struct xlat *x;
+	const struct xlat *x;
 	long addr;
 
 	if (entering(tcp)) {
@@ -2952,7 +3051,7 @@ struct tcb *tcp;
 #endif /* !SVR4 */
 
 #ifdef LINUX
-static struct xlat futexops[] = {
+static const struct xlat futexops[] = {
 	{ FUTEX_WAIT,	"FUTEX_WAIT" },
 	{ FUTEX_WAKE,	"FUTEX_WAKE" },
 	{ FUTEX_FD,	"FUTEX_FD" },
@@ -3022,7 +3121,7 @@ struct tcb *tcp;
     return 0;
 }
 
-static struct xlat schedulers[] = {
+static const struct xlat schedulers[] = {
 	{ SCHED_OTHER,	"SCHED_OTHER" },
 	{ SCHED_RR,	"SCHED_RR" },
 	{ SCHED_FIFO,	"SCHED_FIFO" },
@@ -3052,7 +3151,7 @@ struct tcb *tcp;
 	tprintf("%d, ", (int) tcp->u_arg[0]);
 	printxval(schedulers, tcp->u_arg[1], "SCHED_???");
 	if (umove(tcp, tcp->u_arg[2], &p) < 0)
-	    tprintf(", %lx", tcp->u_arg[2]);
+	    tprintf(", %#lx", tcp->u_arg[2]);
 	else
 	    tprintf(", { %d }", p.__sched_priority);
     }
@@ -3068,7 +3167,7 @@ struct tcb *tcp;
     } else {
 	struct sched_param p;
 	if (umove(tcp, tcp->u_arg[1], &p) < 0)
-	    tprintf("%lx", tcp->u_arg[1]);
+	    tprintf("%#lx", tcp->u_arg[1]);
 	else
 	    tprintf("{ %d }", p.__sched_priority);
     }
@@ -3082,7 +3181,7 @@ struct tcb *tcp;
     if (entering(tcp)) {
 	struct sched_param p;
 	if (umove(tcp, tcp->u_arg[1], &p) < 0)
-	    tprintf("%d, %lx", (int) tcp->u_arg[0], tcp->u_arg[1]);
+	    tprintf("%d, %#lx", (int) tcp->u_arg[0], tcp->u_arg[1]);
 	else
 	    tprintf("%d, { %d }", (int) tcp->u_arg[0], p.__sched_priority);
     }
@@ -3098,4 +3197,39 @@ struct tcb *tcp;
     }
     return 0;
 }
+
+#ifdef X86_64
+#include <asm/prctl.h>
+
+static const struct xlat archvals[] = {
+	{ ARCH_SET_GS,		"ARCH_SET_GS"		},
+	{ ARCH_SET_FS,		"ARCH_SET_FS"		},
+	{ ARCH_GET_FS,		"ARCH_GET_FS"		},
+	{ ARCH_GET_GS,		"ARCH_GET_GS"		},
+	{ 0,			NULL			},
+};
+
+int
+sys_arch_prctl(tcp)
+struct tcb *tcp;
+{
+    if (entering(tcp)) {
+	printxval(archvals, tcp->u_arg[0], "ARCH_???");
+	if (tcp->u_arg[0] == ARCH_SET_GS
+	    || tcp->u_arg[0] == ARCH_SET_FS)
+	    tprintf(", %#lx", tcp->u_arg[1]);
+    } else {
+	if (tcp->u_arg[0] == ARCH_GET_GS
+	    || tcp->u_arg[0] == ARCH_GET_FS) {
+	    long int v;
+	    if (!syserror(tcp) && umove(tcp, tcp->u_arg[1], &v) != -1)
+		tprintf(", [%#lx]", v);
+	    else
+		tprintf(", %#lx", tcp->u_arg[1]);
+	}
+    }
+    return 0;
+}
+#endif
+
 #endif
