@@ -27,7 +27,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: resource.c,v 1.10 2004/04/16 21:58:55 roland Exp $
+ *	$Id: resource.c,v 1.16 2005/12/02 04:08:27 roland Exp $
  */
 
 #include "defs.h"
@@ -59,39 +59,54 @@
 #define sys_setrlimit64	sys_setrlimit
 #endif
 
-static struct xlat resources[] = {
-#ifdef RLIMIT_CPU
-	{ RLIMIT_CPU,	"RLIMIT_CPU"	},
-#endif
-#ifdef RLIMIT_FSIZE
-	{ RLIMIT_FSIZE,	"RLIMIT_FSIZE"	},
-#endif
-#ifdef RLIMIT_DATA
-	{ RLIMIT_DATA,	"RLIMIT_DATA"	},
-#endif
-#ifdef RLIMIT_STACK
-	{ RLIMIT_STACK,	"RLIMIT_STACK"	},
+static const struct xlat resources[] = {
+#ifdef RLIMIT_AS
+	{ RLIMIT_AS,	"RLIMIT_AS"	},
 #endif
 #ifdef RLIMIT_CORE
 	{ RLIMIT_CORE,	"RLIMIT_CORE"	},
 #endif
-#ifdef RLIMIT_RSS
-	{ RLIMIT_RSS,	"RLIMIT_RSS"	},
+#ifdef RLIMIT_CPU
+	{ RLIMIT_CPU,	"RLIMIT_CPU"	},
 #endif
-#ifdef RLIMIT_NPROC
-	{ RLIMIT_NPROC,"RLIMIT_NPROC"	},
+#ifdef RLIMIT_DATA
+	{ RLIMIT_DATA,	"RLIMIT_DATA"	},
 #endif
-#ifdef RLIMIT_NOFILE
-	{ RLIMIT_NOFILE,"RLIMIT_NOFILE"	},
+#ifdef RLIMIT_FSIZE
+	{ RLIMIT_FSIZE,	"RLIMIT_FSIZE"	},
+#endif
+#ifdef RLIMIT_LOCKS
+	{ RLIMIT_LOCKS,	"RLIMIT_LOCKS"	},
 #endif
 #ifdef RLIMIT_MEMLOCK
 	{ RLIMIT_MEMLOCK,	"RLIMIT_MEMLOCK"	},
 #endif
+#ifdef RLIMIT_MSGQUEUE
+	{ RLIMIT_MSGQUEUE,	"RLIMIT_MSGQUEUE"	},
+#endif
+#ifdef RLIMIT_NICE
+	{ RLIMIT_NICE,	"RLIMIT_NICE"	},
+#endif
+#ifdef RLIMIT_NOFILE
+	{ RLIMIT_NOFILE,	"RLIMIT_NOFILE"	},
+#endif
+#ifdef RLIMIT_NPROC
+	{ RLIMIT_NPROC,	"RLIMIT_NPROC"	},
+#endif
+#ifdef RLIMIT_RSS
+	{ RLIMIT_RSS,	"RLIMIT_RSS"	},
+#endif
+#ifdef RLIMIT_RTPRIO
+	{ RLIMIT_RTPRIO,	"RLIMIT_RTPRIO"	},
+#endif
+#ifdef RLIMIT_SIGPENDING
+	{ RLIMIT_SIGPENDING,	"RLIMIT_SIGPENDING"	},
+#endif
+#ifdef RLIMIT_STACK
+	{ RLIMIT_STACK,	"RLIMIT_STACK"	},
+#endif
 #ifdef RLIMIT_VMEM
 	{ RLIMIT_VMEM,	"RLIMIT_VMEM"	},
-#endif
-#ifdef RLIMIT_AS
-	{ RLIMIT_AS,	"RLIMIT_AS"	},
 #endif
 	{ 0,		NULL		},
 };
@@ -220,7 +235,7 @@ struct tcb *tcp;
 
 #ifndef SVR4
 
-static struct xlat usagewho[] = {
+static const struct xlat usagewho[] = {
 	{ RUSAGE_SELF,		"RUSAGE_SELF"		},
 	{ RUSAGE_CHILDREN,	"RUSAGE_CHILDREN"	},
 #ifdef RUSAGE_BOTH
@@ -388,7 +403,7 @@ struct tcb *tcp;
 
 #endif /* LINUX */
 
-static struct xlat priorities[] = {
+static const struct xlat priorities[] = {
 	{ PRIO_PROCESS,	"PRIO_PROCESS"	},
 	{ PRIO_PGRP,	"PRIO_PGRP"	},
 	{ PRIO_USER,	"PRIO_USER"	},
@@ -455,22 +470,23 @@ struct tcb *tcp;
 
 #ifdef LINUX
 
+#define OLD_CMD(c)	((c)<<8)
 #define NEW_CMD(c)      ((0x80<<16)+(c))
 #define XQM_CMD(c)      (('X'<<8)+(c))
 #define NEW_COMMAND(c) (( ((c) >> SUBCMDSHIFT) & (0x80 << 16)))
 #define XQM_COMMAND(c) (( ((c) >> SUBCMDSHIFT) & ('X' << 8)) == ('X' << 8))
 #define OLD_COMMAND(c) (!NEW_COMMAND(c) && !XQM_COMMAND(c))
 
-static struct xlat quotacmds[] = {
-	{ Q_QUOTAON,	"Q_QUOTAON"	},
-	{ Q_QUOTAOFF,	"Q_QUOTAOFF"	},
-	{ Q_GETQUOTA,	"Q_GETQUOTA"	},
-	{ Q_SETQUOTA,	"Q_SETQUOTA"	},
-	{ Q_SETUSE,	"Q_SETUSE"	},
-	{ Q_SYNC,	"Q_SYNC"	},
-	{ Q_SETQLIM,	"Q_SETQLIM"	},
-	{ Q_GETSTATS,	"Q_GETSTATS"	},
-	{ Q_RSQUASH,	"Q_RSQUASH"	},
+static const struct xlat quotacmds[] = {
+	{ OLD_CMD(0x1),	"Q_QUOTAON"	},
+	{ OLD_CMD(0x2),	"Q_QUOTAOFF"	},
+	{ OLD_CMD(0x3),	"Q_GETQUOTA"	},
+	{ OLD_CMD(0x4),	"Q_SETQUOTA"	},
+	{ OLD_CMD(0x5),	"Q_SETUSE"	},
+	{ OLD_CMD(0x6),	"Q_SYNC"	},
+	{ OLD_CMD(0x7),	"Q_SETQLIM"	},
+	{ OLD_CMD(0x8),	"Q_GETSTATS"	},
+	{ OLD_CMD(0x10),"Q_RSQUASH"	},
 	{ NEW_CMD(0x1), "Q_SYNC"        },
 	{ NEW_CMD(0x2), "Q_QUOTAON"     },
 	{ NEW_CMD(0x3), "Q_QUOTAOFF"    },
@@ -488,7 +504,7 @@ static struct xlat quotacmds[] = {
 	{ 0,		NULL		},
 };
 
-static struct xlat quotatypes[] = {
+static const struct xlat quotatypes[] = {
 	{ USRQUOTA,	"USRQUOTA"	},
 	{ GRPQUOTA,	"GRPQUOTA"	},
 	{ 0,		NULL		},
@@ -498,10 +514,18 @@ int
 sys_quotactl(tcp)
 struct tcb *tcp;
 {
+	/*
+	 * The Linux kernel only looks at the low 32 bits of the command
+	 * argument, but on some 64-bit architectures (s390x) this word
+	 * will have been sign-extended when we see it.  The high 1 bits
+	 * don't mean anything, so don't confuse the output with them.
+	 */
+	unsigned int cmd = tcp->u_arg[0];
+
 	if (entering(tcp)) {
-		printxval(quotacmds, (unsigned long) tcp->u_arg[0] >> SUBCMDSHIFT, "Q_???");
+		printxval(quotacmds, cmd >> SUBCMDSHIFT, "Q_???");
 		tprintf("|");
-		printxval(quotatypes, tcp->u_arg[0] & SUBCMDMASK, "???QUOTA");
+		printxval(quotatypes, cmd & SUBCMDMASK, "???QUOTA");
 		tprintf(", ");
 		printstr(tcp, tcp->u_arg[1], -1);
 		tprintf(", %lu, ", tcp->u_arg[2]);
@@ -511,21 +535,29 @@ struct tcb *tcp;
 
 		if (!tcp->u_arg[3])
 			tprintf("NULL");
-               else if (!verbose(tcp) || !OLD_COMMAND(tcp->u_arg[0]))
+               else if (!verbose(tcp) ||
+#ifdef HAVE_STRUCT_DQBLK_DQB_CURBLOCKS
+			!
+#endif
+			OLD_COMMAND(cmd))
 			tprintf("%#lx", tcp->u_arg[3]);
                 else if (umoven(tcp, tcp->u_arg[3], sizeof(struct dqblk),
                     (char *) &dq) < 0)
                         tprintf("???");
 		else {
                         tprintf("{");
-			tprintf("%u, ", dq.dqb_bhardlimit);
-			tprintf("%u, ", dq.dqb_bsoftlimit);
-			tprintf("%u, ", dq.dqb_curblocks);
-			tprintf("%u, ", dq.dqb_ihardlimit);
-			tprintf("%u, ", dq.dqb_isoftlimit);
-			tprintf("%u, ", dq.dqb_curinodes);
-			tprintf("%lu, ", dq.dqb_btime);
-			tprintf("%lu", dq.dqb_itime);
+			tprintf("%llu, ", (unsigned long long) dq.dqb_bhardlimit);
+			tprintf("%llu, ", (unsigned long long) dq.dqb_bsoftlimit);
+#ifdef HAVE_STRUCT_DQBLK_DQB_CURBLOCKS
+			tprintf("%llu, ", (unsigned long long) dq.dqb_curblocks);
+#else
+			tprintf("%llu, ", (unsigned long long) dq.dqb_curspace);
+#endif
+			tprintf("%llu, ", (unsigned long long) dq.dqb_ihardlimit);
+			tprintf("%llu, ", (unsigned long long) dq.dqb_isoftlimit);
+			tprintf("%llu, ", (unsigned long long) dq.dqb_curinodes);
+			tprintf("%llu, ", (unsigned long long) dq.dqb_btime);
+			tprintf("%llu", (unsigned long long) dq.dqb_itime);
                         tprintf("}");
 		}
 
@@ -541,7 +573,7 @@ struct tcb *tcp;
 #include <ufs/ufs/quota.h>
 #endif
 
-static struct xlat quotacmds[] = {
+static const struct xlat quotacmds[] = {
 	{ Q_QUOTAON,	"Q_QUOTAON"	},
 	{ Q_QUOTAOFF,	"Q_QUOTAOFF"	},
 	{ Q_GETQUOTA,	"Q_GETQUOTA"	},
