@@ -27,12 +27,12 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: strace.c,v 1.57 2004/06/04 01:50:45 roland Exp $
+ *	$Id: strace.c,v 1.68 2006/01/12 09:50:49 roland Exp $
  */
 
-#include <sys/types.h>
 #include "defs.h"
 
+#include <sys/types.h>
 #include <signal.h>
 #include <errno.h>
 #include <sys/param.h>
@@ -199,12 +199,12 @@ char *argv[];
 	progname = argv[0];
 	outf = stderr;
 	interactive = 1;
+	set_sortby(DEFAULT_SORTBY);
+	set_personality(DEFAULT_PERSONALITY);
 	qualify("trace=all");
 	qualify("abbrev=all");
 	qualify("verbose=all");
 	qualify("signal=all");
-	set_sortby(DEFAULT_SORTBY);
-	set_personality(DEFAULT_PERSONALITY);
 	while ((c = getopt(argc, argv,
 		"+cdfFhiqrtTvVxza:e:o:O:p:s:S:u:E:")) != EOF) {
 		switch (c) {
@@ -285,6 +285,12 @@ char *argv[];
 			break;
 		case 's':
 			max_strlen = atoi(optarg);
+			if (max_strlen < 0) {
+				fprintf(stderr,
+					"%s: invalid -s argument: %s\n",
+					progname, optarg);
+				exit(1);
+			}
 			break;
 		case 'S':
 			set_sortby(optarg);
@@ -597,6 +603,13 @@ Process %u attached - interrupt to quit\n",
 			}
 			else
 				setreuid(run_uid, run_uid);
+
+			/*
+			 * Induce an immediate stop so that the parent
+			 * will resume us with PTRACE_SYSCALL and display
+			 * this execve call normally.
+			 */
+			kill(getpid(), SIGSTOP);
 #endif /* !USE_PROCFS */
 
 			execv(pathname, &argv[optind]);
@@ -617,9 +630,6 @@ Process %u attached - interrupt to quit\n",
 				exit(1);
 			}
 #endif /* USE_PROCFS */
-#ifndef USE_PROCFS
-			fake_execve(tcp, pathname, &argv[optind], environ);
-#endif /* !USE_PROCFS */
 			break;
 		}
 	}
@@ -992,7 +1002,7 @@ int attaching;
 			if (tcp->status.PR_WHY == PR_SYSENTRY) {
 				tcp->flags &= ~TCB_INSYSCALL;
 				get_scno(tcp);
-				if (tcp->scno == SYS_execve)
+				if (known_scno(tcp) == SYS_execve)
 					break;
 			}
 			/* Set it running: maybe execve will be next. */
@@ -1196,6 +1206,14 @@ int sig;
 	int error = 0;
 #ifdef LINUX
 	int status, resumed;
+	struct tcb *zombie = NULL;
+
+	/* If the group leader is lingering only because of this other
+	   thread now dying, then detach the leader as well.  */
+	if ((tcp->flags & TCB_CLONE_THREAD) &&
+	    tcp->parent->nclone_threads == 1 &&
+	    (tcp->parent->flags & TCB_EXITING))
+		zombie = tcp->parent;
 #endif
 
 	if (tcp->flags & TCB_BPTSET)
@@ -1354,6 +1372,12 @@ int sig;
 		fprintf(stderr, "Process %u detached\n", tcp->pid);
 
 	droptcb(tcp);
+
+#ifdef LINUX
+	if (zombie != NULL)
+		error = detach(zombie) || error;
+#endif
+
 	return error;
 }
 
@@ -1476,7 +1500,7 @@ rebuild_pollv()
 		free (pollv);
 	pollv = (struct pollfd *) malloc(nprocs * sizeof pollv[0]);
 	if (pollv == NULL) {
-		fprintf(stderr, "strace: out of memory for poll vector\n");
+		fprintf(stderr, "%s: out of memory\n", progname);
 		exit(1);
 	}
 
@@ -1926,30 +1950,40 @@ handle_group_exit(struct tcb *tcp, int sig)
 			      ? tcp : NULL);
 
 	if (sig < 0) {
-		if (leader != NULL && leader != tcp)
+		if (leader != NULL && leader != tcp &&
+		    !(leader->flags & TCB_GROUP_EXITING))
 			fprintf(stderr,
 				"PANIC: handle_group_exit: %d leader %d\n",
 				tcp->pid, leader ? leader->pid : -1);
-		droptcb(tcp);	/* Already died.  */
+		detach(tcp);	/* Already died.  */
 	}
 	else {
+		/* Mark that we are taking the process down.  */
+		tcp->flags |= TCB_EXITING | TCB_GROUP_EXITING;
 		if (tcp->flags & TCB_ATTACHED) {
-			if (leader != NULL && leader != tcp) {
-				/* We need to detach the leader so that the
-				   process death will be reported to its real
-				   parent.  But we kill it first to prevent
-				   it doing anything before we kill the whole
-				   process in a moment.  We can use
-				   PTRACE_KILL on a thread that's not already
-				   stopped.  Then the value we pass in
-				   PTRACE_DETACH just sets the death
-				   signal reported to the real parent.  */
-				ptrace(PTRACE_KILL, leader->pid, 0, 0);
-				if (debug)
-					fprintf(stderr,
-						" [%d exit %d kills %d]\n",
-						tcp->pid, sig, leader->pid);
-				detach(leader, sig);
+		  	if (leader != NULL && leader != tcp) {
+				if (leader->flags & TCB_ATTACHED) {
+					/* We need to detach the leader so
+					   that the process death will be
+					   reported to its real parent.
+					   But we kill it first to prevent
+					   it doing anything before we kill
+					   the whole process in a moment.
+					   We can use PTRACE_KILL on a
+					   thread that's not already
+					   stopped.  Then the value we pass
+					   in PTRACE_DETACH just sets the
+					   death signal reported to the
+					   real parent.  */
+					ptrace(PTRACE_KILL, leader->pid, 0, 0);
+					if (debug)
+						fprintf(stderr,
+							" [%d exit %d kills %d]\n",
+							tcp->pid, sig, leader->pid);
+					detach(leader, sig);
+				}
+				else
+					leader->flags |= TCB_GROUP_EXITING;
 			}
 			detach(tcp, sig);
 		}
@@ -1959,6 +1993,8 @@ handle_group_exit(struct tcb *tcp, int sig)
 			return -1;
 		}
 		else {
+			if (leader != NULL)
+				leader->flags |= TCB_GROUP_EXITING;
 			if (leader != NULL && leader != tcp)
 				droptcb(tcp);
 			/* The leader will report to us as parent now,
@@ -1967,18 +2003,6 @@ handle_group_exit(struct tcb *tcp, int sig)
 		}
 	}
 
-	/* Note that TCP and LEADER are no longer valid,
-	   but we can still compare against them.  */
-	if (leader != NULL) {
-		unsigned int i;
-		for (i = 0; i < tcbtabsize; i++) {
-			struct tcb *t = tcbtab[i];
-			if (t != tcp && (t->flags & TCB_CLONE_DETACHED)
-			    && t->parent == leader)
-				droptcb(t);
-		}
-	}
-
 	return 0;
 }
 #endif
@@ -2138,7 +2162,12 @@ Process %d attached (waiting for parent)\n",
 		if (WIFEXITED(status)) {
 			if (debug)
 				fprintf(stderr, "pid %u exited\n", pid);
-			if (tcp->flags & TCB_ATTACHED)
+			if ((tcp->flags & TCB_ATTACHED)
+#ifdef TCB_GROUP_EXITING
+			    && !(tcp->parent && (tcp->parent->flags &
+						 TCB_GROUP_EXITING))
+#endif
+				)
 				fprintf(stderr,
 					"PANIC: attached pid %u exited\n",
 					pid);
@@ -2222,7 +2251,7 @@ Process %d attached (waiting for parent)\n",
 			if (!cflag
 			    && (qual_flags[WSTOPSIG(status)] & QUAL_SIGNAL)) {
 				unsigned long addr = 0, pc = 0;
-#ifdef PT_GETSIGINFO
+#if defined(PT_CR_IPSR) && defined(PT_CR_IIP) && defined(PT_GETSIGINFO)
 #				define PSR_RI	41
 				struct siginfo si;
 				unsigned long psr;
@@ -2233,6 +2262,15 @@ Process %d attached (waiting for parent)\n",
 				pc += (psr >> PSR_RI) & 0x3;
 				ptrace(PT_GETSIGINFO, pid, 0, (long) &si);
 				addr = (unsigned long) si.si_addr;
+#elif defined PTRACE_GETSIGINFO
+				if (WSTOPSIG(status) == SIGSEGV ||
+				    WSTOPSIG(status) == SIGBUS) {
+					siginfo_t si;
+					if (ptrace(PTRACE_GETSIGINFO, pid,
+						   0, &si) == 0)
+						addr = (unsigned long)
+							si.si_addr;
+				}
 #endif
 				printleader(tcp);
 				tprintf("--- %s (%s) @ %lx (%lx) ---",
@@ -2240,7 +2278,8 @@ Process %d attached (waiting for parent)\n",
 					strsignal(WSTOPSIG(status)), pc, addr);
 				printtrailer(tcp);
 			}
-			if ((tcp->flags & TCB_ATTACHED) &&
+			if (((tcp->flags & TCB_ATTACHED) ||
+			     tcp->nclone_threads > 0) &&
 				!sigishandled(tcp, WSTOPSIG(status))) {
 #ifdef TCB_GROUP_EXITING
 				handle_group_exit(tcp, WSTOPSIG(status));
