@@ -27,7 +27,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: system.c,v 1.30 2004/03/01 22:10:54 roland Exp $
+ *	$Id: system.c,v 1.39 2006/01/12 21:21:06 roland Exp $
  */
 
 #include "defs.h"
@@ -36,12 +36,24 @@
 #define _LINUX_SOCKET_H
 #define _LINUX_FS_H
 
-#define MS_RDONLY   1  /* Mount read-only */
-#define MS_NOSUID   2  /* Ignore suid and sgid bits */
-#define MS_NODEV    4  /* Disallow access to device special files */
-#define MS_NOEXEC   8  /* Disallow program execution */
-#define MS_SYNCHRONOUS 16  /* Writes are synced at once */
-#define MS_REMOUNT 32  /* Alter flags of a mounted FS */
+#define MS_RDONLY	 1	/* Mount read-only */
+#define MS_NOSUID	 2	/* Ignore suid and sgid bits */
+#define MS_NODEV	 4	/* Disallow access to device special files */
+#define MS_NOEXEC	 8	/* Disallow program execution */
+#define MS_SYNCHRONOUS	16	/* Writes are synced at once */
+#define MS_REMOUNT	32	/* Alter flags of a mounted FS */
+#define MS_MANDLOCK	64	/* Allow mandatory locks on an FS */
+#define MS_DIRSYNC	128	/* Directory modifications are synchronous */
+#define MS_NOATIME	1024	/* Do not update access times. */
+#define MS_NODIRATIME	2048	/* Do not update directory access times */
+#define MS_BIND		4096
+#define MS_MOVE		8192
+#define MS_REC		16384
+#define MS_VERBOSE	32768
+#define MS_POSIXACL	(1<<16)	/* VFS does not apply the umask */
+#define MS_ACTIVE	(1<<30)
+#define MS_NOUSER	(1<<31)
+#define MS_MGC_VAL	0xc0ed0000	/* Magic flag number */
 
 #include <sys/socket.h>
 #include <netinet/in.h>
@@ -49,17 +61,6 @@
 
 #include <sys/syscall.h>
 
-#ifdef SYS_personality
-/* Workaround for kernel namespace pollution. */
-#define _LINUX_PTRACE_H
-/* Yuck yuck yuck.  We can't include linux/ptrace.h, but personality.h
-   makes a declaration with struct pt_regs, which is defined there. */
-struct pt_regs;
-#define sys_personality kernel_sys_personality
-#include <linux/personality.h>
-#undef sys_personality
-#endif /* SYS_personality */
-
 #ifdef SYS_capget
 #include <linux/capability.h>
 #endif
@@ -78,17 +79,24 @@ struct pt_regs;
 
 #include <linux/sysctl.h>
 
-static struct xlat mount_flags[] = {
+static const struct xlat mount_flags[] = {
+	{ MS_MGC_VAL,	"MS_MGC_VAL"	},
 	{ MS_RDONLY,	"MS_RDONLY"	},
 	{ MS_NOSUID,	"MS_NOSUID"	},
 	{ MS_NODEV,	"MS_NODEV"	},
 	{ MS_NOEXEC,	"MS_NOEXEC"	},
-#ifdef MS_SYNCHRONOUS
 	{ MS_SYNCHRONOUS,"MS_SYNCHRONOUS"},
-#else
-	{ MS_SYNC,	"MS_SYNC"	},
-#endif
 	{ MS_REMOUNT,	"MS_REMOUNT"	},
+	{ MS_MANDLOCK,	"MS_MANDLOCK"	},
+	{ MS_NOATIME,	"MS_NOATIME"	},
+	{ MS_NODIRATIME,"MS_NODIRATIME"	},
+	{ MS_BIND,	"MS_BIND"	},
+	{ MS_MOVE,	"MS_MOVE"	},
+	{ MS_REC,	"MS_REC"	},
+	{ MS_VERBOSE,	"MS_VERBOSE"	},
+	{ MS_POSIXACL,	"MS_POSIXACL"	},
+	{ MS_ACTIVE,	"MS_ACTIVE"	},
+	{ MS_NOUSER,	"MS_NOUSER"	},
 	{ 0,		NULL		},
 };
 
@@ -101,10 +109,17 @@ struct tcb *tcp;
 		tprintf(", ");
 		printpath(tcp, tcp->u_arg[1]);
 		tprintf(", ");
-		printpath(tcp, tcp->u_arg[2]);
+		if ((tcp->u_arg[3] & (MS_BIND|MS_MOVE|MS_REMOUNT)) == 0)
+			printpath(tcp, tcp->u_arg[2]);
+		else
+			tprintf("%#lx", tcp->u_arg[2]);
 		tprintf(", ");
-		printflags(mount_flags, tcp->u_arg[3]);
-		tprintf(", %#lx", tcp->u_arg[4]);
+		printflags(mount_flags, tcp->u_arg[3], "MS_???");
+		tprintf(", ");
+		if ((tcp->u_arg[3] & (MS_BIND|MS_MOVE)) == 0)
+			printstr(tcp, tcp->u_arg[4], -1);
+		else
+			tprintf("%#lx", tcp->u_arg[4]);
 	}
 	return 0;
 }
@@ -124,34 +139,30 @@ struct tcb *tcp;
 	return 0;
 }
 
-static struct xlat personality_options[] = {
-#ifdef PER_LINUX
-	{ PER_LINUX,	"PER_LINUX"	},
-#endif
-#ifdef PER_LINUX_32BIT
-	{ PER_LINUX_32BIT,	"PER_LINUX"	},
-#endif
-#ifdef PER_SVR4
-	{ PER_SVR4,	"PER_SVR4"	},
-#endif
-#ifdef PER_SVR3
-	{ PER_SVR3,	"PER_SVR3"	},
-#endif
-#ifdef PER_SCOSVR3
-	{ PER_SCOSVR3,	"PER_SCOSVR3"	},
-#endif
-#ifdef PER_WYSEV386
-	{ PER_WYSEV386,	"PER_WYSEV386"	},
-#endif
-#ifdef PER_ISCR4
-	{ PER_ISCR4,	"PER_ISCR4"	},
-#endif
-#ifdef PER_BSD
-	{ PER_BSD,	"PER_BSD"	},
-#endif
-#ifdef PER_XENIX
-	{ PER_XENIX,	"PER_XENIX"	},
-#endif
+/* These are not macros, but enums.  We just copy the values by hand
+   from Linux 2.6.9 here.  */
+static const struct xlat personality_options[] = {
+	{ 0,		"PER_LINUX"	},
+	{ 0x00800000,	"PER_LINUX_32BIT"},
+	{ 0x04100001,	"PER_SVR4"	},
+	{ 0x05000002,	"PER_SVR3"	},
+	{ 0x07000003,	"PER_SCOSVR3"	},
+	{ 0x06000003,	"PER_OSR5"	},
+	{ 0x05000004,	"PER_WYSEV386"	},
+	{ 0x04000005,	"PER_ISCR4"	},
+	{ 0x00000006,	"PER_BSD"	},
+	{ 0x04000006,	"PER_SUNOS"	},
+	{ 0x05000007,	"PER_XENIX"	},
+	{ 0x00000008,	"PER_LINUX32"	},
+	{ 0x08000008,	"PER_LINUX32_3GB"},
+	{ 0x04000009,	"PER_IRIX32"	},
+	{ 0x0400000a,	"PER_IRIXN32"	},
+	{ 0x0400000b,	"PER_IRIX64"	},
+	{ 0x0000000c,	"PER_RISCOS"	},
+	{ 0x0400000d,	"PER_SOLARIS"	},
+	{ 0x0410000e,	"PER_UW7"	},
+	{ 0x0000000f,	"PER_OSF4"	},
+	{ 0x00000010,	"PER_HPUX"	},
 	{ 0,		NULL		},
 };
 
@@ -165,19 +176,19 @@ struct tcb *tcp;
 }
 
 #include <linux/reboot.h>
-static struct xlat bootflags1[] = {
+static const struct xlat bootflags1[] = {
 	{ LINUX_REBOOT_MAGIC1,	"LINUX_REBOOT_MAGIC1"	},
 	{ 0,			NULL			},
 };
 
-static struct xlat bootflags2[] = {
+static const struct xlat bootflags2[] = {
 	{ LINUX_REBOOT_MAGIC2,	"LINUX_REBOOT_MAGIC2"	},
 	{ LINUX_REBOOT_MAGIC2A,	"LINUX_REBOOT_MAGIC2A"	},
 	{ LINUX_REBOOT_MAGIC2B,	"LINUX_REBOOT_MAGIC2B"	},
 	{ 0,			NULL			},
 };
 
-static struct xlat bootflags3[] = {
+static const struct xlat bootflags3[] = {
 	{ LINUX_REBOOT_CMD_CAD_OFF,	"LINUX_REBOOT_CMD_CAD_OFF"	},
 	{ LINUX_REBOOT_CMD_RESTART,	"LINUX_REBOOT_CMD_RESTART"	},
 	{ LINUX_REBOOT_CMD_HALT,	"LINUX_REBOOT_CMD_HALT"		},
@@ -192,14 +203,11 @@ sys_reboot(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp)) {
-		if (!printflags(bootflags1, tcp->u_arg[0]))
-			tprintf("LINUX_REBOOT_MAGIC???");
+		printflags(bootflags1, tcp->u_arg[0], "LINUX_REBOOT_MAGIC_???");
 		tprintf(", ");
-		if (!printflags(bootflags2, tcp->u_arg[1]))
-			tprintf("LINUX_REBOOT_MAGIC???");
+		printflags(bootflags2, tcp->u_arg[1], "LINUX_REBOOT_MAGIC_???");
 		tprintf(", ");
-		if (!printflags(bootflags3, tcp->u_arg[2]))
-			tprintf("LINUX_REBOOT_CMD_???");
+		printflags(bootflags3, tcp->u_arg[2], "LINUX_REBOOT_CMD_???");
 		if (tcp->u_arg[2] == LINUX_REBOOT_CMD_RESTART2) {
 			tprintf(", ");
 			printstr(tcp, tcp->u_arg[3], -1);
@@ -209,7 +217,7 @@ struct tcb *tcp;
 }
 
 #ifdef M68K
-static struct xlat cacheflush_scope[] = {
+static const struct xlat cacheflush_scope[] = {
 #ifdef FLUSH_SCOPE_LINE
 	{ FLUSH_SCOPE_LINE,	"FLUSH_SCOPE_LINE" },
 #endif
@@ -222,7 +230,7 @@ static struct xlat cacheflush_scope[] = {
 	{ 0,			NULL },
 };
 
-static struct xlat cacheflush_flags[] = {
+static const struct xlat cacheflush_flags[] = {
 #ifdef FLUSH_CACHE_BOTH
 	{ FLUSH_CACHE_BOTH,	"FLUSH_CACHE_BOTH" },
 #endif
@@ -246,7 +254,7 @@ struct tcb *tcp;
 		printxval(cacheflush_scope, tcp->u_arg[1], "FLUSH_SCOPE_???");
 		tprintf(", ");
 		/* flags */
-		printflags(cacheflush_flags, tcp->u_arg[2]);
+		printflags(cacheflush_flags, tcp->u_arg[2], "FLUSH_CACHE_???");
 		/* len */
 		tprintf(", %lu", tcp->u_arg[3]);
 	}
@@ -277,7 +285,7 @@ struct tcb *tcp;
 	return 0;
 }
 
-static struct xlat bootflags[] = {
+static const struct xlat bootflags[] = {
 	{ RB_AUTOBOOT,	"RB_AUTOBOOT"	},	/* for system auto-booting itself */
 	{ RB_ASKNAME,	"RB_ASKNAME"	},	/* ask for file name to reboot from */
 	{ RB_SINGLE,	"RB_SINGLE"	},	/* reboot to single user only */
@@ -297,8 +305,7 @@ sys_reboot(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp)) {
-		if (!printflags(bootflags, tcp->u_arg[0]))
-			tprintf("RB_???");
+		printflags(bootflags, tcp->u_arg[0], "RB_???");
 		if (tcp->u_arg[0] & RB_STRING) {
 			printstr(tcp, tcp->u_arg[1], -1);
 		}
@@ -336,7 +343,7 @@ struct tcb *tcp;
 	return 0;
 }
 
-static struct xlat mountflags[] = {
+static const struct xlat mountflags[] = {
 	{ M_RDONLY,	"M_RDONLY"	},
 	{ M_NOSUID,	"M_NOSUID"	},
 	{ M_NEWTYPE,	"M_NEWTYPE"	},
@@ -356,7 +363,7 @@ static struct xlat mountflags[] = {
 	{ 0,		NULL		},
 };
 
-static struct xlat nfsflags[] = {
+static const struct xlat nfsflags[] = {
 	{ NFSMNT_SOFT,		"NFSMNT_SOFT"		},
 	{ NFSMNT_WSIZE,		"NFSMNT_WSIZE"		},
 	{ NFSMNT_RSIZE,		"NFSMNT_RSIZE"		},
@@ -396,8 +403,7 @@ struct tcb *tcp;
 		}
 		printstr(tcp, tcp->u_arg[1], -1);
 		tprintf(", ");
-		if (!printflags(mountflags, tcp->u_arg[2] & ~M_NEWTYPE))
-			tprintf("0");
+		printflags(mountflags, tcp->u_arg[2] & ~M_NEWTYPE, "M_???");
 		tprintf(", ");
 
 		if (strcmp(type, "4.2") == 0) {
@@ -417,8 +423,7 @@ struct tcb *tcp;
 			tprintf("[");
 			printsock(tcp, (int) a.addr);
 			tprintf(", ");
-			if (!printflags(nfsflags, a.flags))
-				tprintf("NFSMNT_???");
+			printflags(nfsflags, a.flags, "NFSMNT_???");
 			tprintf(", ws:%u,rs:%u,to:%u,re:%u,",
 				a.wsize, a.rsize, a.timeo, a.retrans);
 			if (a.flags & NFSMNT_HOSTNAME && a.hostname)
@@ -477,7 +482,7 @@ struct tcb *tcp;
 	return printargs(tcp);
 }
 
-static struct xlat ex_auth_flags[] = {
+static const struct xlat ex_auth_flags[] = {
 	{ AUTH_UNIX,	"AUTH_UNIX"	},
 	{ AUTH_DES,	"AUTH_DES"	},
 	{ 0,		NULL		},
@@ -523,7 +528,7 @@ struct tcb *tcp;
 	return 0;
 }
 
-static struct xlat sysconflimits[] = {
+static const struct xlat sysconflimits[] = {
 #ifdef	_SC_ARG_MAX
 	{ _SC_ARG_MAX,	"_SC_ARG_MAX"	},	/* space for argv & envp */
 #endif
@@ -564,7 +569,7 @@ struct tcb *tcp;
 #endif /* SUNOS4 */
 
 #if defined(SUNOS4) || defined(FREEBSD)
-static struct xlat pathconflimits[] = {
+static const struct xlat pathconflimits[] = {
 #ifdef	_PC_LINK_MAX
 	{ _PC_LINK_MAX,	"_PC_LINK_MAX"	},	/* max links to file/dir */
 #endif
@@ -631,7 +636,7 @@ struct tcb *tcp;
 #include <sys/systeminfo.h>
 #include <sys/utsname.h>
 
-static struct xlat sysconfig_options[] = {
+static const struct xlat sysconfig_options[] = {
 #ifdef _CONFIG_NGROUPS
 	{ _CONFIG_NGROUPS,		"_CONFIG_NGROUPS"		},
 #endif
@@ -719,7 +724,7 @@ struct tcb *tcp;
 	return 0;
 }
 
-static struct xlat sysinfo_options[] = {
+static const struct xlat sysinfo_options[] = {
 	{ SI_SYSNAME,		"SI_SYSNAME"		},
 	{ SI_HOSTNAME,		"SI_HOSTNAME"		},
 	{ SI_RELEASE,		"SI_RELEASE"		},
@@ -767,7 +772,7 @@ struct tcb *tcp;
 
 #include <sys/syssgi.h>
 
-static struct xlat syssgi_options[] = {
+static const struct xlat syssgi_options[] = {
 	{ SGI_SYSID,		"SGI_SYSID"		},
 #ifdef SGI_RDUBLK
 	{ SGI_RDUBLK,		"SGI_RDUBLK"		},
@@ -883,247 +888,247 @@ static struct xlat syssgi_options[] = {
 #ifdef SGI_GET_FP_PRECISE
 	{ SGI_GET_FP_PRECISE,	"SGI_GET_FP_PRECISE"	},
 #endif
-#ifdef SGI_GET_CONFIG_SMM	
+#ifdef SGI_GET_CONFIG_SMM
 	{ SGI_GET_CONFIG_SMM,	"SGI_GET_CONFIG_SMM"	},
 #endif
-#ifdef SGI_FP_IMPRECISE_SUPP	
+#ifdef SGI_FP_IMPRECISE_SUPP
 	{ SGI_FP_IMPRECISE_SUPP,"SGI_FP_IMPRECISE_SUPP"	},
 #endif
-#ifdef SGI_CONFIG_NSMM_SUPP	
+#ifdef SGI_CONFIG_NSMM_SUPP
 	{ SGI_CONFIG_NSMM_SUPP,	"SGI_CONFIG_NSMM_SUPP"	},
 #endif
-#ifdef SGI_RT_TSTAMP_CREATE    
+#ifdef SGI_RT_TSTAMP_CREATE
 	{ SGI_RT_TSTAMP_CREATE,	"SGI_RT_TSTAMP_CREATE"	},
 #endif
-#ifdef SGI_RT_TSTAMP_DELETE    
+#ifdef SGI_RT_TSTAMP_DELETE
 	{ SGI_RT_TSTAMP_DELETE,	"SGI_RT_TSTAMP_DELETE"	},
 #endif
-#ifdef SGI_RT_TSTAMP_START     
+#ifdef SGI_RT_TSTAMP_START
 	{ SGI_RT_TSTAMP_START,	"SGI_RT_TSTAMP_START"	},
 #endif
-#ifdef SGI_RT_TSTAMP_STOP      
+#ifdef SGI_RT_TSTAMP_STOP
 	{ SGI_RT_TSTAMP_STOP,	"SGI_RT_TSTAMP_STOP"	},
 #endif
-#ifdef SGI_RT_TSTAMP_ADDR      
+#ifdef SGI_RT_TSTAMP_ADDR
 	{ SGI_RT_TSTAMP_ADDR,	"SGI_RT_TSTAMP_ADDR"	},
 #endif
-#ifdef SGI_RT_TSTAMP_MASK      
+#ifdef SGI_RT_TSTAMP_MASK
 	{ SGI_RT_TSTAMP_MASK,	"SGI_RT_TSTAMP_MASK"	},
 #endif
-#ifdef SGI_RT_TSTAMP_EOB_MODE  
+#ifdef SGI_RT_TSTAMP_EOB_MODE
 	{ SGI_RT_TSTAMP_EOB_MODE,"SGI_RT_TSTAMP_EOB_MODE"},
 #endif
-#ifdef SGI_USE_FP_BCOPY	
+#ifdef SGI_USE_FP_BCOPY
 	{ SGI_USE_FP_BCOPY,	"SGI_USE_FP_BCOPY"	},
 #endif
-#ifdef SGI_GET_UST		
+#ifdef SGI_GET_UST
 	{ SGI_GET_UST,		"SGI_GET_UST"		},
 #endif
-#ifdef SGI_SPECULATIVE_EXEC	
+#ifdef SGI_SPECULATIVE_EXEC
 	{ SGI_SPECULATIVE_EXEC,	"SGI_SPECULATIVE_EXEC"	},
 #endif
-#ifdef SGI_XLV_NEXT_RQST	
+#ifdef SGI_XLV_NEXT_RQST
 	{ SGI_XLV_NEXT_RQST,	"SGI_XLV_NEXT_RQST"	},
 #endif
-#ifdef SGI_XLV_ATTR_CURSOR	
+#ifdef SGI_XLV_ATTR_CURSOR
 	{ SGI_XLV_ATTR_CURSOR,	"SGI_XLV_ATTR_CURSOR"	},
 #endif
-#ifdef SGI_XLV_ATTR_GET	
+#ifdef SGI_XLV_ATTR_GET
 	{ SGI_XLV_ATTR_GET,	"SGI_XLV_ATTR_GET"	},
 #endif
-#ifdef SGI_XLV_ATTR_SET	
+#ifdef SGI_XLV_ATTR_SET
 	{ SGI_XLV_ATTR_SET,	"SGI_XLV_ATTR_SET"	},
 #endif
 #ifdef SGI_BTOOLSIZE
 	{ SGI_BTOOLSIZE,	"SGI_BTOOLSIZE"		},
 #endif
-#ifdef SGI_BTOOLGET		
+#ifdef SGI_BTOOLGET
 	{ SGI_BTOOLGET,		"SGI_BTOOLGET"		},
 #endif
-#ifdef SGI_BTOOLREINIT		
+#ifdef SGI_BTOOLREINIT
 	{ SGI_BTOOLREINIT,	"SGI_BTOOLREINIT"	},
 #endif
-#ifdef SGI_CREATE_UUID		
+#ifdef SGI_CREATE_UUID
 	{ SGI_CREATE_UUID,	"SGI_CREATE_UUID"	},
 #endif
-#ifdef SGI_NOFPE		
+#ifdef SGI_NOFPE
 	{ SGI_NOFPE,		"SGI_NOFPE"		},
 #endif
-#ifdef SGI_OLD_SOFTFP		
+#ifdef SGI_OLD_SOFTFP
 	{ SGI_OLD_SOFTFP,	"SGI_OLD_SOFTFP"	},
 #endif
-#ifdef SGI_FS_INUMBERS		
+#ifdef SGI_FS_INUMBERS
 	{ SGI_FS_INUMBERS,	"SGI_FS_INUMBERS"	},
 #endif
-#ifdef SGI_FS_BULKSTAT		
+#ifdef SGI_FS_BULKSTAT
 	{ SGI_FS_BULKSTAT,	"SGI_FS_BULKSTAT"	},
 #endif
-#ifdef SGI_RT_TSTAMP_WAIT	
+#ifdef SGI_RT_TSTAMP_WAIT
 	{ SGI_RT_TSTAMP_WAIT,	"SGI_RT_TSTAMP_WAIT"	},
 #endif
-#ifdef SGI_RT_TSTAMP_UPDATE    
+#ifdef SGI_RT_TSTAMP_UPDATE
 	{ SGI_RT_TSTAMP_UPDATE,	"SGI_RT_TSTAMP_UPDATE"	},
 #endif
-#ifdef SGI_PATH_TO_HANDLE	
+#ifdef SGI_PATH_TO_HANDLE
 	{ SGI_PATH_TO_HANDLE,	"SGI_PATH_TO_HANDLE"	},
 #endif
-#ifdef SGI_PATH_TO_FSHANDLE	
+#ifdef SGI_PATH_TO_FSHANDLE
 	{ SGI_PATH_TO_FSHANDLE,	"SGI_PATH_TO_FSHANDLE"	},
 #endif
-#ifdef SGI_FD_TO_HANDLE	
+#ifdef SGI_FD_TO_HANDLE
 	{ SGI_FD_TO_HANDLE,	"SGI_FD_TO_HANDLE"	},
 #endif
-#ifdef SGI_OPEN_BY_HANDLE	
+#ifdef SGI_OPEN_BY_HANDLE
 	{ SGI_OPEN_BY_HANDLE,	"SGI_OPEN_BY_HANDLE"	},
 #endif
-#ifdef SGI_READLINK_BY_HANDLE	
+#ifdef SGI_READLINK_BY_HANDLE
 	{ SGI_READLINK_BY_HANDLE,"SGI_READLINK_BY_HANDLE"},
 #endif
-#ifdef SGI_READ_DANGID		
+#ifdef SGI_READ_DANGID
 	{ SGI_READ_DANGID,	"SGI_READ_DANGID"	},
 #endif
-#ifdef SGI_CONST		
+#ifdef SGI_CONST
 	{ SGI_CONST,		"SGI_CONST"		},
 #endif
-#ifdef SGI_XFS_FSOPERATIONS	
+#ifdef SGI_XFS_FSOPERATIONS
 	{ SGI_XFS_FSOPERATIONS,	"SGI_XFS_FSOPERATIONS"	},
 #endif
-#ifdef SGI_SETASH		
+#ifdef SGI_SETASH
 	{ SGI_SETASH,		"SGI_SETASH"		},
 #endif
-#ifdef SGI_GETASH		
+#ifdef SGI_GETASH
 	{ SGI_GETASH,		"SGI_GETASH"		},
 #endif
-#ifdef SGI_SETPRID		
+#ifdef SGI_SETPRID
 	{ SGI_SETPRID,		"SGI_SETPRID"		},
 #endif
-#ifdef SGI_GETPRID		
+#ifdef SGI_GETPRID
 	{ SGI_GETPRID,		"SGI_GETPRID"		},
 #endif
-#ifdef SGI_SETSPINFO		
+#ifdef SGI_SETSPINFO
 	{ SGI_SETSPINFO,	"SGI_SETSPINFO"		},
 #endif
-#ifdef SGI_GETSPINFO		
+#ifdef SGI_GETSPINFO
 	{ SGI_GETSPINFO,	"SGI_GETSPINFO"		},
 #endif
-#ifdef SGI_SHAREII		
+#ifdef SGI_SHAREII
 	{ SGI_SHAREII,		"SGI_SHAREII"		},
 #endif
-#ifdef SGI_NEWARRAYSESS	
+#ifdef SGI_NEWARRAYSESS
 	{ SGI_NEWARRAYSESS,	"SGI_NEWARRAYSESS"	},
 #endif
-#ifdef SGI_GETDFLTPRID		
+#ifdef SGI_GETDFLTPRID
 	{ SGI_GETDFLTPRID,	"SGI_GETDFLTPRID"	},
 #endif
-#ifdef SGI_SET_DISMISSED_EXC_CNT 
+#ifdef SGI_SET_DISMISSED_EXC_CNT
 	{ SGI_SET_DISMISSED_EXC_CNT,"SGI_SET_DISMISSED_EXC_CNT"	},
 #endif
-#ifdef SGI_GET_DISMISSED_EXC_CNT 
+#ifdef SGI_GET_DISMISSED_EXC_CNT
 	{ SGI_GET_DISMISSED_EXC_CNT,"SGI_GET_DISMISSED_EXC_CNT"	},
 #endif
-#ifdef SGI_CYCLECNTR_SIZE	
+#ifdef SGI_CYCLECNTR_SIZE
 	{ SGI_CYCLECNTR_SIZE,	"SGI_CYCLECNTR_SIZE"	},
 #endif
-#ifdef SGI_QUERY_FASTTIMER	
+#ifdef SGI_QUERY_FASTTIMER
 	{ SGI_QUERY_FASTTIMER,	"SGI_QUERY_FASTTIMER"	},
 #endif
-#ifdef SGI_PIDSINASH		
+#ifdef SGI_PIDSINASH
 	{ SGI_PIDSINASH,	"SGI_PIDSINASH"		},
 #endif
-#ifdef SGI_ULI			
+#ifdef SGI_ULI
 	{ SGI_ULI,		"SGI_ULI"		},
 #endif
-#ifdef SGI_LPG_SHMGET          
+#ifdef SGI_LPG_SHMGET
 	{ SGI_LPG_SHMGET,	"SGI_LPG_SHMGET"	},
 #endif
-#ifdef SGI_LPG_MAP             
+#ifdef SGI_LPG_MAP
 	{ SGI_LPG_MAP,		"SGI_LPG_MAP"		},
 #endif
-#ifdef SGI_CACHEFS_SYS		
+#ifdef SGI_CACHEFS_SYS
 	{ SGI_CACHEFS_SYS,	"SGI_CACHEFS_SYS"	},
 #endif
-#ifdef SGI_NFSNOTIFY		
+#ifdef SGI_NFSNOTIFY
 	{ SGI_NFSNOTIFY,	"SGI_NFSNOTIFY"		},
 #endif
-#ifdef SGI_LOCKDSYS		
+#ifdef SGI_LOCKDSYS
 	{ SGI_LOCKDSYS,		"SGI_LOCKDSYS"		},
 #endif
-#ifdef SGI_EVENTCTR            
+#ifdef SGI_EVENTCTR
 	{ SGI_EVENTCTR,		"SGI_EVENTCTR"		},
 #endif
-#ifdef SGI_GETPRUSAGE          
+#ifdef SGI_GETPRUSAGE
 	{ SGI_GETPRUSAGE,	"SGI_GETPRUSAGE"	},
 #endif
-#ifdef SGI_PROCMASK_LOCATION	
+#ifdef SGI_PROCMASK_LOCATION
 	{ SGI_PROCMASK_LOCATION,"SGI_PROCMASK_LOCATION"	},
 #endif
-#ifdef SGI_UNUSED		
+#ifdef SGI_UNUSED
 	{ SGI_UNUSED,		"SGI_UNUSED"		},
 #endif
-#ifdef SGI_CKPT_SYS		
+#ifdef SGI_CKPT_SYS
 	{ SGI_CKPT_SYS,		"SGI_CKPT_SYS"		},
 #endif
-#ifdef SGI_CKPT_SYS		
+#ifdef SGI_CKPT_SYS
 	{ SGI_CKPT_SYS,		"SGI_CKPT_SYS"		},
 #endif
-#ifdef SGI_GETGRPPID		
+#ifdef SGI_GETGRPPID
 	{ SGI_GETGRPPID,	"SGI_GETGRPPID"		},
 #endif
-#ifdef SGI_GETSESPID		
+#ifdef SGI_GETSESPID
 	{ SGI_GETSESPID,	"SGI_GETSESPID"		},
 #endif
-#ifdef SGI_ENUMASHS		
+#ifdef SGI_ENUMASHS
 	{ SGI_ENUMASHS,		"SGI_ENUMASHS"		},
 #endif
-#ifdef SGI_SETASMACHID		
+#ifdef SGI_SETASMACHID
 	{ SGI_SETASMACHID,	"SGI_SETASMACHID"	},
 #endif
-#ifdef SGI_GETASMACHID		
+#ifdef SGI_GETASMACHID
 	{ SGI_GETASMACHID,	"SGI_GETASMACHID"	},
 #endif
-#ifdef SGI_GETARSESS		
+#ifdef SGI_GETARSESS
 	{ SGI_GETARSESS,	"SGI_GETARSESS"		},
 #endif
-#ifdef SGI_JOINARRAYSESS	
+#ifdef SGI_JOINARRAYSESS
 	{ SGI_JOINARRAYSESS,	"SGI_JOINARRAYSESS"	},
 #endif
-#ifdef SGI_SPROC_KILL		
+#ifdef SGI_SPROC_KILL
 	{ SGI_SPROC_KILL,	"SGI_SPROC_KILL"	},
 #endif
-#ifdef SGI_DBA_CONFIG		
+#ifdef SGI_DBA_CONFIG
 	{ SGI_DBA_CONFIG,	"SGI_DBA_CONFIG"	},
 #endif
-#ifdef SGI_RELEASE_NAME	
+#ifdef SGI_RELEASE_NAME
 	{ SGI_RELEASE_NAME,	"SGI_RELEASE_NAME"	},
 #endif
-#ifdef SGI_SYNCH_CACHE_HANDLER 
+#ifdef SGI_SYNCH_CACHE_HANDLER
 	{ SGI_SYNCH_CACHE_HANDLER,"SGI_SYNCH_CACHE_HANDLER"},
 #endif
-#ifdef SGI_SWASH_INIT		
+#ifdef SGI_SWASH_INIT
 	{ SGI_SWASH_INIT,	"SGI_SWASH_INIT"	},
 #endif
-#ifdef SGI_NUMA_MIGR_PAGE	
+#ifdef SGI_NUMA_MIGR_PAGE
 	{ SGI_NUMA_MIGR_PAGE,	"SGI_NUMA_MIGR_PAGE"	},
 #endif
-#ifdef SGI_NUMA_MIGR_PAGE_ALT	
+#ifdef SGI_NUMA_MIGR_PAGE_ALT
 	{ SGI_NUMA_MIGR_PAGE_ALT,"SGI_NUMA_MIGR_PAGE_ALT"},
 #endif
-#ifdef SGI_KAIO_USERINIT	
+#ifdef SGI_KAIO_USERINIT
 	{ SGI_KAIO_USERINIT,	"SGI_KAIO_USERINIT"	},
 #endif
-#ifdef SGI_KAIO_READ		
+#ifdef SGI_KAIO_READ
 	{ SGI_KAIO_READ,	"SGI_KAIO_READ"		},
 #endif
-#ifdef SGI_KAIO_WRITE		
+#ifdef SGI_KAIO_WRITE
 	{ SGI_KAIO_WRITE,	"SGI_KAIO_WRITE"	},
 #endif
-#ifdef SGI_KAIO_SUSPEND	
+#ifdef SGI_KAIO_SUSPEND
 	{ SGI_KAIO_SUSPEND,	"SGI_KAIO_SUSPEND"	},
 #endif
-#ifdef SGI_KAIO_STATS		
+#ifdef SGI_KAIO_STATS
 	{ SGI_KAIO_STATS,	"SGI_KAIO_STATS"	},
 #endif
-#ifdef SGI_INITIAL_PT_SPROC	
+#ifdef SGI_INITIAL_PT_SPROC
 	{ SGI_INITIAL_PT_SPROC,	"SGI_INITIAL_PT_SPROC"	},
 #endif
 	{ 0,			NULL			},
@@ -1156,7 +1161,7 @@ struct uio;
 #include <sys/fs/nfs.h>
 #include <sys/fs/nfs_clnt.h>
 
-static struct xlat mount_flags[] = {
+static const struct xlat mount_flags[] = {
 	{ MS_RDONLY,	"MS_RDONLY"	},
 	{ MS_FSS,	"MS_FSS"	},
 	{ MS_DATA,	"MS_DATA"	},
@@ -1170,7 +1175,7 @@ static struct xlat mount_flags[] = {
 	{ 0,		NULL		},
 };
 
-static struct xlat nfs_flags[] = {
+static const struct xlat nfs_flags[] = {
 	{ NFSMNT_SOFT,		"NFSMNT_SOFT"		},
 	{ NFSMNT_WSIZE,		"NFSMNT_WSIZE"		},
 	{ NFSMNT_RSIZE,		"NFSMNT_RSIZE"		},
@@ -1211,7 +1216,7 @@ struct tcb *tcp;
 		tprintf(", ");
 		printpath(tcp, tcp->u_arg[1]);
 		tprintf(", ");
-		printflags(mount_flags, tcp->u_arg[2]);
+		printflags(mount_flags, tcp->u_arg[2], "MS_???");
 		if (tcp->u_arg[2] & (MS_FSS | MS_DATA)) {
 			tprintf(", ");
 			tprintf("%ld", tcp->u_arg[3]);
@@ -1228,8 +1233,7 @@ struct tcb *tcp;
 					tprintf("addr=");
 					printsock(tcp, (int) args.addr);
 					tprintf(", flags=");
-					if (!printflags(nfs_flags, args.flags))
-						tprintf("NFSMNT_???");
+					printflags(nfs_flags, args.flags, "NFSMNT_???");
 					tprintf(", hostname=");
 					printstr(tcp, (int) args.hostname, -1);
 					tprintf(", ...}");
@@ -1257,7 +1261,7 @@ struct tcb *tcp;
 
 #include <sys/fs/vx_ioctl.h>
 
-static struct xlat mount_flags[] = {
+static const struct xlat mount_flags[] = {
 	{ MS_RDONLY,	"MS_RDONLY"	},
 	{ MS_FSS,	"MS_FSS"	},
 	{ MS_DATA,	"MS_DATA"	},
@@ -1271,7 +1275,7 @@ static struct xlat mount_flags[] = {
 };
 
 #ifdef VX_MS_MASK
-static struct xlat vxfs_flags[] = {
+static const struct xlat vxfs_flags[] = {
 	{ VX_MS_NOLOG,		"VX_MS_NOLOG"		},
 	{ VX_MS_BLKCLEAR,	"VX_MS_BLKCLEAR"	},
 	{ VX_MS_SNAPSHOT,	"VX_MS_SNAPSHOT"	},
@@ -1293,7 +1297,7 @@ static struct xlat vxfs_flags[] = {
 };
 #endif
 
-static struct xlat nfs_flags[] = {
+static const struct xlat nfs_flags[] = {
 	{ NFSMNT_SOFT,		"NFSMNT_SOFT"		},
 	{ NFSMNT_WSIZE,		"NFSMNT_WSIZE"		},
 	{ NFSMNT_RSIZE,		"NFSMNT_RSIZE"		},
@@ -1324,7 +1328,7 @@ struct tcb *tcp;
 		tprintf(", ");
 		printpath(tcp, tcp->u_arg[1]);
 		tprintf(", ");
-		printflags(mount_flags, tcp->u_arg[2]);
+		printflags(mount_flags, tcp->u_arg[2], "MS_???");
 		/* The doc sez that the file system type is given as a
 		   fsindex, and we should use sysfs to work out the name.
 		   This appears to be untrue for UW.  Maybe it's untrue
@@ -1348,12 +1352,11 @@ struct tcb *tcp;
 					tprintf("%#lx", tcp->u_arg[4]);
 				else {
 					tprintf("{ flags=");
-					if (!printflags(vxfs_flags, args.mflags))
-						tprintf("0x%08x", args.mflags);
+					printflags(vxfs_flags, args.mflags, "VX_MS_???");
 					if (args.mflags & VX_MS_SNAPSHOT) {
 						tprintf (", snapof=");
 						printstr (tcp,
-							  (long) args.primaryspec, 
+							  (long) args.primaryspec,
 							  -1);
 						if (args.snapsize > 0)
 							tprintf (", snapsize=%ld", args.snapsize);
@@ -1382,8 +1385,7 @@ struct tcb *tcp;
 						printsock(tcp, (int) addr.buf, addr.len);
 					}
 					tprintf(", flags=");
-					if (!printflags(nfs_flags, args.flags))
-						tprintf("NFSMNT_???");
+					printflags(nfs_flags, args.flags, "NFSMNT_???");
 					tprintf(", hostname=");
 					printstr(tcp, (int) args.hostname, -1);
 					tprintf(", ...}");
@@ -1419,7 +1421,7 @@ struct tcb *tcp;
 
 #ifdef SYS_capget
 
-static struct xlat capabilities[] = {
+static const struct xlat capabilities[] = {
 	{ 1<<CAP_CHOWN,		"CAP_CHOWN"	},
 	{ 1<<CAP_DAC_OVERRIDE,	"CAP_DAC_OVERRIDE"},
 	{ 1<<CAP_DAC_READ_SEARCH,"CAP_DAC_READ_SEARCH"},
@@ -1461,14 +1463,14 @@ struct tcb *tcp;
 	if(!entering(tcp)) {
 		if (!arg0) {
 			if ((arg0 = malloc(sizeof(*arg0))) == NULL) {
-				fprintf(stderr, "sys_capget: no memory\n");
+				fprintf(stderr, "out of memory\n");
 				tprintf("%#lx, %#lx", tcp->u_arg[0], tcp->u_arg[1]);
 				return -1;
 			}
 		}
 		if (!arg1) {
 			if ((arg1 = malloc(sizeof(*arg1))) == NULL) {
-				fprintf(stderr, "sys_capget: no memory\n");
+				fprintf(stderr, "out of memory\n");
 				tprintf("%#lx, %#lx", tcp->u_arg[0], tcp->u_arg[1]);
 				return -1;
 			}
@@ -1492,13 +1494,13 @@ struct tcb *tcp;
 			tprintf("???");
 		else {
 			tprintf("{");
-			printflags(capabilities, arg1->effective);
+			printflags(capabilities, arg1->effective, "CAP_???");
 			tprintf(", ");
-			printflags(capabilities, arg1->permitted);
+			printflags(capabilities, arg1->permitted, "CAP_???");
 			tprintf(", ");
-			printflags(capabilities, arg1->inheritable);
+			printflags(capabilities, arg1->inheritable, "CAP_???");
 			tprintf("}");
-		} 
+		}
 	}
 	return 0;
 }
@@ -1513,14 +1515,14 @@ struct tcb *tcp;
 	if(entering(tcp)) {
 		if (!arg0) {
 			if ((arg0 = malloc(sizeof(*arg0))) == NULL) {
-				fprintf(stderr, "sys_capset: no memory\n");
+				fprintf(stderr, "out of memory\n");
 				tprintf("%#lx, %#lx", tcp->u_arg[0], tcp->u_arg[1]);
 				return -1;
 			}
 		}
 		if (!arg1) {
 			if ((arg1 = malloc(sizeof(*arg1))) == NULL) {
-				fprintf(stderr, "sys_capset: no memory\n");
+				fprintf(stderr, "out of memory\n");
 				tprintf("%#lx, %#lx", tcp->u_arg[0], tcp->u_arg[1]);
 				return -1;
 			}
@@ -1544,13 +1546,13 @@ struct tcb *tcp;
 			tprintf("???");
 		else {
 			tprintf("{");
-			printflags(capabilities, arg1->effective);
+			printflags(capabilities, arg1->effective, "CAP_???");
 			tprintf(", ");
-			printflags(capabilities, arg1->permitted);
+			printflags(capabilities, arg1->permitted, "CAP_???");
 			tprintf(", ");
-			printflags(capabilities, arg1->inheritable);
+			printflags(capabilities, arg1->inheritable, "CAP_???");
 			tprintf("}");
-		} 
+		}
 	}
 	return 0;
 }
@@ -1572,7 +1574,7 @@ struct tcb *tcp;
 #endif
 
 #ifdef LINUX
-static struct xlat sysctl_root[] = {
+static const struct xlat sysctl_root[] = {
 	{ CTL_KERN, "CTL_KERN" },
 	{ CTL_VM, "CTL_VM" },
 	{ CTL_NET, "CTL_NET" },
@@ -1583,7 +1585,7 @@ static struct xlat sysctl_root[] = {
 	{ 0, NULL }
 };
 
-static struct xlat sysctl_kern[] = {
+static const struct xlat sysctl_kern[] = {
 	{ KERN_OSTYPE, "KERN_OSTYPE" },
 	{ KERN_OSRELEASE, "KERN_OSRELEASE" },
 	{ KERN_OSREV, "KERN_OSREV" },
@@ -1625,7 +1627,7 @@ static struct xlat sysctl_kern[] = {
 	{ 0, NULL }
 };
 
-static struct xlat sysctl_vm[] = {
+static const struct xlat sysctl_vm[] = {
 #ifdef VM_SWAPCTL
 	{ VM_SWAPCTL, "VM_SWAPCTL" },
 #endif
@@ -1679,7 +1681,7 @@ static struct xlat sysctl_vm[] = {
 	{ 0, NULL },
 };
 
-static struct xlat sysctl_net[] = {
+static const struct xlat sysctl_net[] = {
 	{ NET_CORE, "NET_CORE" },
 	{ NET_ETHER, "NET_ETHER" },
 	{ NET_802, "NET_802" },
@@ -1698,7 +1700,7 @@ static struct xlat sysctl_net[] = {
 	{ 0, NULL }
 };
 
-static struct xlat sysctl_net_core[] = {
+static const struct xlat sysctl_net_core[] = {
 	{ NET_CORE_WMEM_MAX, "NET_CORE_WMEM_MAX" },
 	{ NET_CORE_RMEM_MAX, "NET_CORE_RMEM_MAX" },
 	{ NET_CORE_WMEM_DEFAULT, "NET_CORE_WMEM_DEFAULT" },
@@ -1711,13 +1713,13 @@ static struct xlat sysctl_net_core[] = {
 	{ 0, NULL }
 };
 
-static struct xlat sysctl_net_unix[] = {
+static const struct xlat sysctl_net_unix[] = {
 	{ NET_UNIX_DESTROY_DELAY, "NET_UNIX_DESTROY_DELAY" },
 	{ NET_UNIX_DELETE_DELAY, "NET_UNIX_DELETE_DELAY" },
 	{ 0, NULL }
 };
 
-static struct xlat sysctl_net_ipv4[] = {
+static const struct xlat sysctl_net_ipv4[] = {
 	{ NET_IPV4_FORWARD, "NET_IPV4_FORWARD" },
 	{ NET_IPV4_DYNADDR, "NET_IPV4_DYNADDR" },
 	{ NET_IPV4_CONF, "NET_IPV4_CONF" },
@@ -1760,7 +1762,7 @@ static struct xlat sysctl_net_ipv4[] = {
 	{  0, NULL }
 };
 
-static struct xlat sysctl_net_ipv4_route[] = {
+static const struct xlat sysctl_net_ipv4_route[] = {
 	{ NET_IPV4_ROUTE_FLUSH, "NET_IPV4_ROUTE_FLUSH" },
 	{ NET_IPV4_ROUTE_MIN_DELAY, "NET_IPV4_ROUTE_MIN_DELAY" },
 	{ NET_IPV4_ROUTE_MAX_DELAY, "NET_IPV4_ROUTE_MAX_DELAY" },
@@ -1778,7 +1780,7 @@ static struct xlat sysctl_net_ipv4_route[] = {
 	{ 0, NULL }
 };
 
-static struct xlat sysctl_net_ipv4_conf[] = {
+static const struct xlat sysctl_net_ipv4_conf[] = {
 	{ NET_IPV4_CONF_FORWARDING, "NET_IPV4_CONF_FORWARDING" },
 	{ NET_IPV4_CONF_MC_FORWARDING, "NET_IPV4_CONF_MC_FORWARDING" },
 	{ NET_IPV4_CONF_PROXY_ARP, "NET_IPV4_CONF_PROXY_ARP" },
@@ -1793,14 +1795,14 @@ static struct xlat sysctl_net_ipv4_conf[] = {
 	{ 0, NULL }
 };
 
-static struct xlat sysctl_net_ipv6[] = {
+static const struct xlat sysctl_net_ipv6[] = {
 	{ NET_IPV6_CONF, "NET_IPV6_CONF" },
 	{ NET_IPV6_NEIGH, "NET_IPV6_NEIGH" },
 	{ NET_IPV6_ROUTE, "NET_IPV6_ROUTE" },
 	{ 0, NULL }
 };
 
-static struct xlat sysctl_net_ipv6_route[] = {
+static const struct xlat sysctl_net_ipv6_route[] = {
 	{ NET_IPV6_ROUTE_FLUSH, "NET_IPV6_ROUTE_FLUSH" },
 	{ NET_IPV6_ROUTE_GC_THRESH, "NET_IPV6_ROUTE_GC_THRESH" },
 	{ NET_IPV6_ROUTE_MAX_SIZE, "NET_IPV6_ROUTE_MAX_SIZE" },
@@ -1817,13 +1819,25 @@ struct tcb *tcp;
 {
 	struct __sysctl_args info;
 	int *name;
-	umove (tcp, tcp->u_arg[0], &info);
+	unsigned long size;
 
-	name = alloca (sizeof (int) * info.nlen);
-	umoven(tcp, (size_t) info.name, sizeof (int) * info.nlen, (char *) name);
+	if (umove (tcp, tcp->u_arg[0], &info) < 0)
+		return printargs(tcp);
+
+	size = sizeof (int) * (unsigned long) info.nlen;
+	name = (size / sizeof (int) != info.nlen) ? NULL : malloc (size);
+	if (name == NULL ||
+	    umoven(tcp, (unsigned long) info.name, size, (char *) name) < 0) {
+		free(name);
+		if (entering(tcp))
+			tprintf("{%p, %d, %p, %p, %p, %Zu}",
+				info.name, info.nlen, info.oldval,
+				info.oldlenp, info.newval, info.newlen);
+		return 0;
+	}
 
 	if (entering(tcp)) {
-		int cnt = 0;
+		int cnt = 0, max_cnt;
 
 		tprintf("{{");
 
@@ -1913,13 +1927,18 @@ struct tcb *tcp;
 			goto out;
 		}
 	out:
-		while (cnt < info.nlen)
+		max_cnt = info.nlen;
+		if (abbrev(tcp) && max_cnt > max_strlen)
+			max_cnt = max_strlen;
+		while (cnt < max_cnt)
 			tprintf(", %x", name[cnt++]);
+		if (cnt < info.nlen)
+			tprintf(", ...");
 		tprintf("}, %d, ", info.nlen);
 	} else {
 		size_t oldlen;
-		umove(tcp, (size_t)info.oldlenp, &oldlen);
-		if (info.nlen >= 2
+		if (umove(tcp, (size_t)info.oldlenp, &oldlen) >= 0
+		    && info.nlen >= 2
 		    && ((name[0] == CTL_KERN
 			 && (name[1] == KERN_OSRELEASE
 			     || name[1] == KERN_OSTYPE
@@ -1945,6 +1964,8 @@ struct tcb *tcp;
 		}
 		tprintf("}");
 	}
+
+	free(name);
 	return 0;
 }
 #else
@@ -1965,7 +1986,7 @@ struct tcb *tcp;
 	char ctl[1024];
 	size_t len;
 	int i, numeric;
-	
+
 	if (entering(tcp)) {
 		if (tcp->u_arg[1] < 0 || tcp->u_arg[1] > CTL_MAXNAME ||
 		    (umoven(tcp, tcp->u_arg[0], tcp->u_arg[1] * sizeof(int),
@@ -1996,7 +2017,7 @@ struct tcb *tcp;
 		if (!syserror(tcp) && (umove(tcp, tcp->u_arg[3], &len) >= 0)) {
 			printstr(tcp, tcp->u_arg[2], len);
 			tprintf(", [%u], ", len);
-		} else 
+		} else
 			tprintf("%#lx, %#lx, ", tcp->u_arg[2], tcp->u_arg[3]);
 		printstr(tcp, tcp->u_arg[4], tcp->u_arg[5]);
 		tprintf(", %lu", tcp->u_arg[5]);
@@ -2010,10 +2031,10 @@ struct tcb *tcp;
 #include <sys/ksym.h>
 #include <sys/elf.h>
 
-static struct xlat ksym_flags[] = {
-	{ STT_NOTYPE,	"STT_NOTYPE"	},	
-	{ STT_FUNC,	"STT_FUNC"	},	
-	{ STT_OBJECT,	"STT_OBJECT"	},	
+static const struct xlat ksym_flags[] = {
+	{ STT_NOTYPE,	"STT_NOTYPE"	},
+	{ STT_FUNC,	"STT_FUNC"	},
+	{ STT_OBJECT,	"STT_OBJECT"	},
 	{ 0,		NULL		},
 };
 
@@ -2053,7 +2074,7 @@ struct tcb *tcp;
 struct cred;
 #include <sys/nscsys.h>
 
-static struct xlat ssi_cmd [] = {
+static const struct xlat ssi_cmd [] = {
 	{ SSISYS_BADOP,	"SSISYS_BADOP"	},
 	{ SSISYS_LDLVL_INIT,"SSISYS_LDLVL_INIT"},
 	{ SSISYS_LDLVL_GETVEC,"SSISYS_LDLVL_GETVEC"},
@@ -2150,7 +2171,7 @@ struct tcb *tcp;
 			if (iov.tio_udatainlen != sizeof cni ||
 			    umove (tcp, (long) iov.tio_udatain, &cni) < 0)
 				goto bad_out;
-			if (cni.info_len != sizeof info || 
+			if (cni.info_len != sizeof info ||
 			    iov.tio_udataoutlen != sizeof &info ||
 			    umove (tcp, (long) iov.tio_udataout, &info) < 0)
 				goto bad_out;
@@ -2158,7 +2179,7 @@ struct tcb *tcp;
 				 info.node_num, info.node_totalcpus,
 				 info.node_onlinecpus);
 			break;
-			
+
 		    default:
 		    bad_out:
 			if (iov.tio_udataoutlen) {
@@ -2182,7 +2203,7 @@ struct tcb *tcp;
 #define __NEW_UTS_LEN 64
 #endif
 
-static struct xlat sysmips_operations[] = {
+static const struct xlat sysmips_operations[] = {
 	{ SETNAME,		"SETNAME"	},
 	{ FLUSH_CACHE,		"FLUSH_CACHE"	},
 	{ MIPS_FIXADE,		"MIPS_FIXADE"	},
